@@ -11,12 +11,12 @@ import aiosqlite
 from sqlspec.adapters.aiosqlite.core import (
     build_insert_statement,
     collect_rows,
+    create_mapped_exception,
     default_statement_config,
     driver_profile,
     format_identifier,
     normalize_execute_many_parameters,
     normalize_execute_parameters,
-    raise_exception,
     resolve_rowcount,
 )
 from sqlspec.adapters.aiosqlite.data_dictionary import AiosqliteDataDictionary
@@ -88,12 +88,8 @@ class AiosqliteExceptionHandler:
         if exc_val is None:
             return False
         if isinstance(exc_val, (aiosqlite.Error, sqlite3.Error)):
-            try:
-                raise_exception(exc_val)
-            except Exception as mapped:
-                self.pending_exception = mapped
-                return True
-            return False
+            self.pending_exception = create_mapped_exception(exc_val)
+            return True
         return False
 
 
@@ -256,14 +252,20 @@ class AiosqliteDriver(AsyncDriverAdapterBase):
         arrow_table = self._coerce_arrow_table(source)
         if overwrite:
             statement = f"DELETE FROM {format_identifier(table)}"
-            async with self.handle_database_exceptions(), self.with_cursor(self.connection) as cursor:
+            exc_handler = self.handle_database_exceptions()
+            async with exc_handler, self.with_cursor(self.connection) as cursor:
                 await cursor.execute(statement)
+            if exc_handler.pending_exception is not None:
+                raise exc_handler.pending_exception from None
 
         columns, records = self._arrow_table_to_rows(arrow_table)
         if records:
             insert_sql = build_insert_statement(table, columns)
-            async with self.handle_database_exceptions(), self.with_cursor(self.connection) as cursor:
+            exc_handler = self.handle_database_exceptions()
+            async with exc_handler, self.with_cursor(self.connection) as cursor:
                 await cursor.executemany(insert_sql, records)
+            if exc_handler.pending_exception is not None:
+                raise exc_handler.pending_exception from None
 
         telemetry_payload = self._build_ingest_telemetry(arrow_table)
         telemetry_payload["destination"] = table
