@@ -13,11 +13,11 @@ from asyncmy.cursors import Cursor, DictCursor  # pyright: ignore
 from sqlspec.adapters.asyncmy.core import (
     build_insert_statement,
     collect_rows,
+    create_mapped_exception,
     default_statement_config,
     detect_json_columns,
     driver_profile,
     format_identifier,
-    map_exception,
     normalize_execute_many_parameters,
     normalize_execute_parameters,
     normalize_lastrowid,
@@ -95,13 +95,11 @@ class AsyncmyExceptionHandler:
         if exc_type is None:
             return False
         if issubclass(exc_type, asyncmy.errors.Error):
-            try:
-                result = map_exception(exc_val, logger=logger)
-                if result is True:
-                    return True
-            except Exception as mapped:
-                self.pending_exception = mapped
+            result = create_mapped_exception(exc_val, logger=logger)
+            if result is True:
                 return True
+            self.pending_exception = cast("Exception", result)
+            return True
         return False
 
 
@@ -322,14 +320,20 @@ class AsyncmyDriver(AsyncDriverAdapterBase):
         arrow_table = self._coerce_arrow_table(source)
         if overwrite:
             statement = f"TRUNCATE TABLE {format_identifier(table)}"
-            async with self.handle_database_exceptions(), self.with_cursor(self.connection) as cursor:
+            exc_handler = self.handle_database_exceptions()
+            async with exc_handler, self.with_cursor(self.connection) as cursor:
                 await cursor.execute(statement)
+            if exc_handler.pending_exception is not None:
+                raise exc_handler.pending_exception from None
 
         columns, records = self._arrow_table_to_rows(arrow_table)
         if records:
             insert_sql = build_insert_statement(table, columns)
-            async with self.handle_database_exceptions(), self.with_cursor(self.connection) as cursor:
+            exc_handler = self.handle_database_exceptions()
+            async with exc_handler, self.with_cursor(self.connection) as cursor:
                 await cursor.executemany(insert_sql, records)
+            if exc_handler.pending_exception is not None:
+                raise exc_handler.pending_exception from None
 
         telemetry_payload = self._build_ingest_telemetry(arrow_table)
         telemetry_payload["destination"] = table
