@@ -18,12 +18,12 @@ from sqlspec.adapters.bigquery.core import (
     build_load_job_telemetry,
     build_retry,
     collect_rows,
+    create_mapped_exception,
     default_statement_config,
     detect_emulator,
     driver_profile,
     is_simple_insert,
     normalize_script_rowcount,
-    raise_exception,
     run_query_job,
     storage_api_available,
     try_bulk_insert,
@@ -114,11 +114,8 @@ class BigQueryExceptionHandler:
         if exc_type is None:
             return False
         if issubclass(exc_type, GoogleCloudError):
-            try:
-                raise_exception(exc_val)
-            except Exception as mapped:
-                self.pending_exception = mapped
-                return True
+            self.pending_exception = create_mapped_exception(exc_val)
+            return True
         return False
 
 
@@ -416,7 +413,10 @@ class BigQueryDriver(SyncDriverAdapterBase):
         # Get compiled SQL and parameters
         sql, driver_params = self._get_compiled_sql(prepared_statement, config)
 
-        with self.handle_database_exceptions():
+        exc_handler = self.handle_database_exceptions()
+        arrow_result: ArrowResult | None = None
+
+        with exc_handler:
             query_job = run_query_job(
                 self.connection,
                 sql,
@@ -430,15 +430,22 @@ class BigQueryDriver(SyncDriverAdapterBase):
             # Native Arrow via Storage API
             arrow_table = query_job.to_arrow()
 
-            return build_arrow_result_from_table(
+            arrow_result = build_arrow_result_from_table(
                 prepared_statement,
                 arrow_table,
                 return_format=return_format,
                 batch_size=batch_size,
                 arrow_schema=arrow_schema,
             )
-        msg = "Unreachable"
-        raise RuntimeError(msg)  # pragma: no cover
+
+        if exc_handler.pending_exception is not None:
+            raise exc_handler.pending_exception from None
+
+        if arrow_result is None:
+            msg = "Unreachable"
+            raise RuntimeError(msg)  # pragma: no cover
+
+        return arrow_result
 
     # ─────────────────────────────────────────────────────────────────────────────
     # STORAGE API METHODS
