@@ -20,6 +20,7 @@ from sqlspec.adapters.psycopg.core import (
     build_pipeline_execution_result,
     build_truncate_command,
     collect_rows,
+    create_mapped_exception,
     default_statement_config,
     driver_profile,
     execute_with_optional_parameters,
@@ -27,7 +28,6 @@ from sqlspec.adapters.psycopg.core import (
     executemany_or_skip,
     executemany_or_skip_async,
     pipeline_supported,
-    raise_exception,
     resolve_rowcount,
 )
 from sqlspec.adapters.psycopg.data_dictionary import PsycopgAsyncDataDictionary, PsycopgSyncDataDictionary
@@ -153,11 +153,8 @@ class PsycopgSyncExceptionHandler:
         if exc_type is None:
             return False
         if issubclass(exc_type, psycopg.Error):
-            try:
-                raise_exception(exc_val)
-            except Exception as mapped:
-                self.pending_exception = mapped
-                return True
+            self.pending_exception = create_mapped_exception(exc_val)
+            return True
         return False
 
 
@@ -387,6 +384,10 @@ class PsycopgSyncDriver(PsycopgPipelineMixin, SyncDriverAdapterBase):
     def _execute_stack_pipeline(
         self, stack: "StatementStack", prepared_ops: "list[PreparedStackOperation]"
     ) -> "tuple[StackResult, ...]":
+        def _raise_pending_exception(exception_ctx: "PsycopgSyncExceptionHandler") -> None:
+            if exception_ctx.pending_exception is not None:
+                raise exception_ctx.pending_exception from None
+
         results: list[StackResult] = []
         started_transaction = False
 
@@ -396,12 +397,14 @@ class PsycopgSyncDriver(PsycopgPipelineMixin, SyncDriverAdapterBase):
                     self.begin()
                     started_transaction = True
 
+                exception_handlers = []
                 with ExitStack() as resource_stack:
                     pipeline = resource_stack.enter_context(self.connection.pipeline())
                     pending: list[PipelineCursorEntry] = []
 
                     for prepared in prepared_ops:
                         exception_ctx = self.handle_database_exceptions()
+                        exception_handlers.append(exception_ctx)
                         resource_stack.enter_context(exception_ctx)
                         cursor = resource_stack.enter_context(self.with_cursor(self.connection))
 
@@ -431,6 +434,9 @@ class PsycopgSyncDriver(PsycopgPipelineMixin, SyncDriverAdapterBase):
                         sql_result = self.build_statement_result(statement, execution_result)
                         results.append(StackResult.from_sql_result(sql_result))
 
+                for exception_ctx in exception_handlers:
+                    _raise_pending_exception(exception_ctx)
+
                 if started_transaction:
                     self.commit()
             except Exception:
@@ -485,17 +491,23 @@ class PsycopgSyncDriver(PsycopgPipelineMixin, SyncDriverAdapterBase):
         arrow_table = self._coerce_arrow_table(source)
         if overwrite:
             truncate_sql = build_truncate_command(table)
-            with self.with_cursor(self.connection) as cursor, self.handle_database_exceptions():
+            exc_handler = self.handle_database_exceptions()
+            with self.with_cursor(self.connection) as cursor, exc_handler:
                 cursor.execute(truncate_sql)
+            if exc_handler.pending_exception is not None:
+                raise exc_handler.pending_exception from None
         columns, records = self._arrow_table_to_rows(arrow_table)
         if records:
             copy_sql = build_copy_from_command(table, columns)
+            exc_handler = self.handle_database_exceptions()
             with ExitStack() as stack:
-                stack.enter_context(self.handle_database_exceptions())
+                stack.enter_context(exc_handler)
                 cursor = stack.enter_context(self.with_cursor(self.connection))
                 copy_ctx = stack.enter_context(cursor.copy(copy_sql))
                 for record in records:
                     copy_ctx.write_row(record)
+            if exc_handler.pending_exception is not None:
+                raise exc_handler.pending_exception from None
         telemetry_payload = self._build_ingest_telemetry(arrow_table)
         telemetry_payload["destination"] = table
         self._attach_partition_telemetry(telemetry_payload, partitioner)
@@ -581,11 +593,8 @@ class PsycopgAsyncExceptionHandler:
         if exc_type is None:
             return False
         if issubclass(exc_type, psycopg.Error):
-            try:
-                raise_exception(exc_val)
-            except Exception as mapped:
-                self.pending_exception = mapped
-                return True
+            self.pending_exception = create_mapped_exception(exc_val)
+            return True
         return False
 
 
@@ -824,6 +833,10 @@ class PsycopgAsyncDriver(PsycopgPipelineMixin, AsyncDriverAdapterBase):
     async def _execute_stack_pipeline(
         self, stack: "StatementStack", prepared_ops: "list[PreparedStackOperation]"
     ) -> "tuple[StackResult, ...]":
+        def _raise_pending_exception(exception_ctx: "PsycopgAsyncExceptionHandler") -> None:
+            if exception_ctx.pending_exception is not None:
+                raise exception_ctx.pending_exception from None
+
         results: list[StackResult] = []
         started_transaction = False
 
@@ -833,12 +846,14 @@ class PsycopgAsyncDriver(PsycopgPipelineMixin, AsyncDriverAdapterBase):
                     await self.begin()
                     started_transaction = True
 
+                exception_handlers = []
                 async with AsyncExitStack() as resource_stack:
                     pipeline = await resource_stack.enter_async_context(self.connection.pipeline())
                     pending: list[PipelineCursorEntry] = []
 
                     for prepared in prepared_ops:
                         exception_ctx = self.handle_database_exceptions()
+                        exception_handlers.append(exception_ctx)
                         await resource_stack.enter_async_context(exception_ctx)
                         cursor = await resource_stack.enter_async_context(self.with_cursor(self.connection))
 
@@ -868,6 +883,9 @@ class PsycopgAsyncDriver(PsycopgPipelineMixin, AsyncDriverAdapterBase):
                         sql_result = self.build_statement_result(statement, execution_result)
                         results.append(StackResult.from_sql_result(sql_result))
 
+                for exception_ctx in exception_handlers:
+                    _raise_pending_exception(exception_ctx)
+
                 if started_transaction:
                     await self.commit()
             except Exception:
@@ -922,17 +940,23 @@ class PsycopgAsyncDriver(PsycopgPipelineMixin, AsyncDriverAdapterBase):
         arrow_table = self._coerce_arrow_table(source)
         if overwrite:
             truncate_sql = build_truncate_command(table)
-            async with self.with_cursor(self.connection) as cursor, self.handle_database_exceptions():
+            exc_handler = self.handle_database_exceptions()
+            async with self.with_cursor(self.connection) as cursor, exc_handler:
                 await cursor.execute(truncate_sql)
+            if exc_handler.pending_exception is not None:
+                raise exc_handler.pending_exception from None
         columns, records = self._arrow_table_to_rows(arrow_table)
         if records:
             copy_sql = build_copy_from_command(table, columns)
+            exc_handler = self.handle_database_exceptions()
             async with AsyncExitStack() as stack:
-                await stack.enter_async_context(self.handle_database_exceptions())
+                await stack.enter_async_context(exc_handler)
                 cursor = await stack.enter_async_context(self.with_cursor(self.connection))
                 copy_ctx = await stack.enter_async_context(cursor.copy(copy_sql))
                 for record in records:
                     await copy_ctx.write_row(record)
+            if exc_handler.pending_exception is not None:
+                raise exc_handler.pending_exception from None
         telemetry_payload = self._build_ingest_telemetry(arrow_table)
         telemetry_payload["destination"] = table
         self._attach_partition_telemetry(telemetry_payload, partitioner)
