@@ -4,9 +4,11 @@ Provides abstract base classes and core functionality for SQL query builders.
 """
 
 import hashlib
+import re
 import uuid
 from abc import ABC, abstractmethod
-from typing import TYPE_CHECKING, Any, NoReturn, cast
+from collections.abc import Callable, Iterable, Mapping
+from typing import Any, NoReturn, cast
 
 import sqlglot
 from sqlglot import Dialect, exp
@@ -15,10 +17,12 @@ from sqlglot.errors import ParseError as SQLGlotParseError
 from sqlglot.optimizer import optimize
 from typing_extensions import Self
 
+from sqlspec.builder._vector_expressions import VectorDistance
 from sqlspec.core import (
     SQL,
     ParameterStyle,
     ParameterStyleConfig,
+    SQLResult,
     StatementConfig,
     get_cache,
     get_cache_config,
@@ -26,19 +30,63 @@ from sqlspec.core import (
 )
 from sqlspec.exceptions import SQLBuilderError
 from sqlspec.utils.logging import get_logger
-from sqlspec.utils.type_guards import has_expression_and_parameters, has_with_method, is_expression
+from sqlspec.utils.type_guards import has_expression_and_parameters, has_name, has_with_method, is_expression
 
-if TYPE_CHECKING:
-    from sqlspec.core import SQLResult
-
-__all__ = ("QueryBuilder", "SafeQuery")
+__all__ = ("BuiltQuery", "ExpressionBuilder", "QueryBuilder")
 
 MAX_PARAMETER_COLLISION_ATTEMPTS = 1000
+PARAMETER_INDEX_PATTERN = re.compile(r"^param_(?P<index>\d+)$")
+
+
+class _ExpressionParameterizer:
+    __slots__ = ("_builder",)
+
+    def __init__(self, builder: "QueryBuilder") -> None:
+        self._builder = builder
+
+    def __call__(self, node: exp.Expression) -> exp.Expression:
+        if isinstance(node, exp.Literal):
+            if node.this in {True, False, None}:
+                return node
+
+            parent = node.parent
+            if isinstance(parent, exp.Array) and node.find_ancestor(VectorDistance) is not None:
+                return node
+
+            value = node.this
+            if node.is_number and isinstance(node.this, str):
+                try:
+                    value = float(node.this) if "." in node.this or "e" in node.this.lower() else int(node.this)
+                except ValueError:
+                    value = node.this
+
+            param_name = self._builder.add_parameter_for_expression(value, context="where")
+            return exp.Placeholder(this=param_name)
+        return node
+
+
+class _PlaceholderReplacer:
+    __slots__ = ("_param_mapping",)
+
+    def __init__(self, param_mapping: dict[str, str]) -> None:
+        self._param_mapping = param_mapping
+
+    def __call__(self, node: exp.Expression) -> exp.Expression:
+        if isinstance(node, exp.Placeholder) and str(node.this) in self._param_mapping:
+            return exp.Placeholder(this=self._param_mapping[str(node.this)])
+        return node
+
+
+def _unquote_identifier(node: exp.Expression) -> exp.Expression:
+    if isinstance(node, exp.Identifier):
+        node.set("quoted", False)
+    return node
+
 
 logger = get_logger(__name__)
 
 
-class SafeQuery:
+class BuiltQuery:
     """SQL query with bound parameters."""
 
     __slots__ = ("dialect", "parameters", "sql")
@@ -48,6 +96,18 @@ class SafeQuery:
         self.parameters = parameters if parameters is not None else {}
         self.dialect = dialect
 
+    def __repr__(self) -> str:
+        parameter_keys = sorted(self.parameters.keys())
+        return f"BuiltQuery(sql={self.sql!r}, parameters={parameter_keys!r}, dialect={self.dialect!r})"
+
+    def __eq__(self, other: object) -> bool:
+        if not isinstance(other, BuiltQuery):
+            return NotImplemented
+        return self.sql == other.sql and self.parameters == other.parameters and self.dialect == other.dialect
+
+    def __hash__(self) -> int:
+        return hash((self.sql, frozenset(self.parameters.items()), self.dialect))
+
 
 class QueryBuilder(ABC):
     """Abstract base class for SQL query builders.
@@ -58,7 +118,10 @@ class QueryBuilder(ABC):
 
     __slots__ = (
         "_expression",
+        "_lock_targets_quoted",
+        "_merge_target_quoted",
         "_parameter_counter",
+        "_parameter_name_counters",
         "_parameters",
         "_with_ctes",
         "dialect",
@@ -86,9 +149,29 @@ class QueryBuilder(ABC):
         self.simplify_expressions = simplify_expressions
 
         self._expression: exp.Expression | None = None
+        self._parameter_name_counters: dict[str, int] = {}
         self._parameters: dict[str, Any] = {}
         self._parameter_counter: int = 0
         self._with_ctes: dict[str, exp.CTE] = {}
+        self._lock_targets_quoted = False
+        self._merge_target_quoted = False
+
+    @classmethod
+    def _parse_query_builder_kwargs(
+        cls, kwargs: "dict[str, Any]"
+    ) -> "tuple[DialectType | None, dict[str, dict[str, str]] | None, bool, bool, bool, bool]":
+        dialect = kwargs.pop("dialect", None)
+        schema = kwargs.pop("schema", None)
+        enable_optimization = kwargs.pop("enable_optimization", True)
+        optimize_joins = kwargs.pop("optimize_joins", True)
+        optimize_predicates = kwargs.pop("optimize_predicates", True)
+        simplify_expressions = kwargs.pop("simplify_expressions", True)
+
+        if kwargs:
+            unknown = ", ".join(sorted(kwargs.keys()))
+            cls._raise_sql_builder_error(f"Unexpected QueryBuilder arguments: {unknown}")
+
+        return (dialect, schema, enable_optimization, optimize_joins, optimize_predicates, simplify_expressions)
 
     def _initialize_expression(self) -> None:
         """Initialize the base expression. Called after __init__."""
@@ -207,7 +290,7 @@ class QueryBuilder(ABC):
         if self._expression is None:
             self._raise_sql_builder_error("QueryBuilder expression not initialized.")
 
-        base_expression = self._expression.copy() if copy else self._expression
+        base_expression = self._expression.copy() if copy or self._with_ctes else self._expression
 
         if not self._with_ctes:
             return base_expression
@@ -285,6 +368,18 @@ class QueryBuilder(ABC):
         self._parameters[param_name] = value
         return param_name
 
+    def add_parameter_for_expression(self, value: Any, context: str | None = None) -> str:
+        """Add a parameter for expression parameterization.
+
+        Args:
+            value: The value of the parameter.
+            context: Optional context hint for parameter naming.
+
+        Returns:
+            Parameter placeholder name.
+        """
+        return self._add_parameter(value, context=context)
+
     def _parameterize_expression(self, expression: exp.Expression) -> exp.Expression:
         """Replace literal values in an expression with bound parameters.
 
@@ -299,15 +394,7 @@ class QueryBuilder(ABC):
             A new expression with literals replaced by parameter placeholders
         """
 
-        def replacer(node: exp.Expression) -> exp.Expression:
-            if isinstance(node, exp.Literal):
-                if node.this in {True, False, None}:
-                    return node
-                param_name = self._add_parameter(node.this, context="where")
-                return exp.Placeholder(this=param_name)
-            return node
-
-        return expression.transform(replacer, copy=False)
+        return expression.transform(_ExpressionParameterizer(self), copy=False)
 
     def add_parameter(self: Self, value: Any, name: str | None = None) -> tuple[Self, str]:
         """Explicitly adds a parameter to the query.
@@ -334,6 +421,52 @@ class QueryBuilder(ABC):
         self._parameters[param_name] = value
         return self, param_name
 
+    def load_parameters(self, parameters: "Mapping[str, Any]") -> None:
+        """Load a parameter mapping into the builder.
+
+        Args:
+            parameters: Mapping of parameter names to values.
+
+        Raises:
+            SQLBuilderError: If a parameter name already exists on the builder.
+        """
+        if not parameters:
+            return
+
+        for name, value in parameters.items():
+            if name in self._parameters:
+                self._raise_sql_builder_error(f"Parameter name '{name}' already exists.")
+            self._parameters[name] = value
+            self._update_parameter_counter(name)
+
+    def load_ctes(self, ctes: "Iterable[exp.CTE]") -> None:
+        """Load SQLGlot CTE nodes into the builder.
+
+        Args:
+            ctes: Iterable of CTE expressions to register.
+
+        Raises:
+            SQLBuilderError: If a CTE alias is missing or duplicated.
+        """
+        for cte in ctes:
+            alias = self._resolve_cte_alias(cte)
+            if alias in self._with_ctes:
+                self._raise_sql_builder_error(f"CTE '{alias}' already exists.")
+            self._with_ctes[alias] = cte
+
+    def _resolve_cte_alias(self, cte: exp.CTE) -> str:
+        alias_name = cte.alias_or_name
+        if not alias_name:
+            self._raise_sql_builder_error("CTE alias is required.")
+        return str(alias_name)
+
+    def _update_parameter_counter(self, name: str) -> None:
+        match = PARAMETER_INDEX_PATTERN.match(name)
+        if not match:
+            return
+        index = int(match.group("index"))
+        self._parameter_counter = max(self._parameter_counter, index)
+
     def _generate_unique_parameter_name(self, base_name: str) -> str:
         """Generate unique parameter name when collision occurs.
 
@@ -343,15 +476,42 @@ class QueryBuilder(ABC):
         Returns:
             A unique parameter name that doesn't exist in current parameters
         """
+        current_index = self._parameter_name_counters.get(base_name, 0)
+
         if base_name not in self._parameters:
+            # First use keeps the base name, counter stays at 0
+            self._parameter_name_counters[base_name] = current_index
             return base_name
 
-        for i in range(1, MAX_PARAMETER_COLLISION_ATTEMPTS):
-            name = f"{base_name}_{i}"
-            if name not in self._parameters:
-                return name
+        next_index = current_index + 1
+        candidate = f"{base_name}_{next_index}"
 
-        return f"{base_name}_{uuid.uuid4().hex[:8]}"
+        while candidate in self._parameters:
+            next_index += 1
+            if next_index > MAX_PARAMETER_COLLISION_ATTEMPTS:
+                return f"{base_name}_{uuid.uuid4().hex[:8]}"
+            candidate = f"{base_name}_{next_index}"
+
+        self._parameter_name_counters[base_name] = next_index
+        return candidate
+
+    def _create_placeholder(self, value: Any, base_name: str) -> tuple[exp.Placeholder, str]:
+        """Backwards-compatible placeholder helper (delegates to create_placeholder)."""
+        return self.create_placeholder(value, base_name)
+
+    def create_placeholder(self, value: Any, base_name: str) -> tuple[exp.Placeholder, str]:
+        """Create placeholder expression with a unique parameter name.
+
+        Args:
+            value: Parameter value to bind.
+            base_name: Seed for parameter naming.
+
+        Returns:
+            Tuple of placeholder expression and the final parameter name.
+        """
+        param_name = self._generate_unique_parameter_name(base_name)
+        _, param_name = self.add_parameter(value, name=param_name)
+        return exp.Placeholder(this=param_name), param_name
 
     def _merge_cte_parameters(self, cte_name: str, parameters: dict[str, Any]) -> dict[str, str]:
         """Merge CTE parameters with unique naming to prevent collisions.
@@ -383,12 +543,7 @@ class QueryBuilder(ABC):
             Updated expression with new placeholder names
         """
 
-        def placeholder_replacer(node: exp.Expression) -> exp.Expression:
-            if isinstance(node, exp.Placeholder) and str(node.this) in param_mapping:
-                return exp.Placeholder(this=param_mapping[str(node.this)])
-            return node
-
-        return expression.transform(placeholder_replacer, copy=False)
+        return expression.transform(_PlaceholderReplacer(param_mapping), copy=False)
 
     def _generate_builder_cache_key(self, config: "StatementConfig | None" = None) -> str:
         """Generate cache key based on builder state and configuration.
@@ -405,13 +560,15 @@ class QueryBuilder(ABC):
             self._expression = self._create_base_expression()
 
         expr_sql: str = self._expression.sql() if self._expression else "None"
+        parameters_snapshot = sorted(self._parameters.items())
+        parameters_hash = hashlib.sha256(str(parameters_snapshot).encode()).hexdigest()[:8]
 
         state_parts = [
             f"expression:{expr_sql}",
-            f"parameters:{sorted(self._parameters.items())}",
+            f"parameters_hash:{parameters_hash}",
             f"ctes:{sorted(self._with_ctes.keys())}",
             f"dialect:{dialect_name}",
-            f"schema:{self.schema}",
+            f"schema_hash:{hashlib.sha256(str(self.schema).encode()).hexdigest()[:8]}",
             f"optimization:{self.enable_optimization}",
             f"optimize_joins:{self.optimize_joins}",
             f"optimize_predicates:{self.optimize_predicates}",
@@ -451,7 +608,7 @@ class QueryBuilder(ABC):
         self._with_ctes[alias] = exp.CTE(this=cte_select_expression, alias=exp.to_table(alias))
         return self
 
-    def build(self, dialect: DialectType = None) -> "SafeQuery":
+    def build(self, dialect: DialectType = None) -> "BuiltQuery":
         """Builds the SQL query string and parameters.
 
         Args:
@@ -459,7 +616,7 @@ class QueryBuilder(ABC):
                     instead of the builder's default dialect.
 
         Returns:
-            SafeQuery: A dataclass containing the SQL string and parameters.
+            BuiltQuery: A dataclass containing the SQL string and parameters.
 
         Examples:
             # Use builder's default dialect
@@ -479,15 +636,21 @@ class QueryBuilder(ABC):
 
         try:
             if isinstance(final_expression, exp.Expression):
-                sql_string = final_expression.sql(dialect=target_dialect, pretty=True)
+                normalized_expression = (
+                    self._unquote_identifiers_for_oracle(final_expression)
+                    if self._is_oracle_dialect(target_dialect)
+                    else final_expression
+                )
+                identify = self._should_identify(target_dialect)
+                sql_string = normalized_expression.sql(dialect=target_dialect, pretty=True, identify=identify)
+                sql_string = self._strip_lock_identifier_quotes(sql_string)
             else:
                 sql_string = str(final_expression)
         except Exception as e:
             err_msg = f"Error generating SQL from expression: {e!s}"
-            logger.exception("SQL generation failed")
             self._raise_sql_builder_error(err_msg, e)
 
-        return SafeQuery(sql=sql_string, parameters=self._parameters.copy(), dialect=dialect or self.dialect)
+        return BuiltQuery(sql=sql_string, parameters=self._parameters.copy(), dialect=dialect or self.dialect)
 
     def to_sql(self, show_parameters: bool = False, dialect: DialectType = None) -> str:
         """Return SQL string with optional parameter substitution.
@@ -553,6 +716,9 @@ class QueryBuilder(ABC):
         if not self.enable_optimization:
             return expression
 
+        if not self.optimize_joins and not self.optimize_predicates and not self.simplify_expressions:
+            return expression
+
         optimizer_settings = {
             "optimize_joins": self.optimize_joins,
             "pushdown_predicates": self.optimize_predicates,
@@ -565,7 +731,7 @@ class QueryBuilder(ABC):
         )
 
         cache = get_cache()
-        cached_optimized = cache.get("optimized", cache_key)
+        cached_optimized = cache.get_optimized(cache_key)
         if cached_optimized:
             return cast("exp.Expression", cached_optimized)
 
@@ -573,7 +739,7 @@ class QueryBuilder(ABC):
             optimized = optimize(
                 expression, schema=self.schema, dialect=self.dialect_name, optimizer_settings=optimizer_settings
             )
-            cache.put("optimized", cache_key, optimized)
+            cache.put_optimized(cache_key, optimized)
         except Exception:
             logger.debug("Expression optimization failed, using original expression")
             return expression
@@ -596,12 +762,12 @@ class QueryBuilder(ABC):
         cache_key_str = self._generate_builder_cache_key(config)
 
         cache = get_cache()
-        cached_sql = cache.get("builder", cache_key_str)
+        cached_sql = cache.get_builder(cache_key_str)
         if cached_sql is not None:
             return cast("SQL", cached_sql)
 
         sql_statement = self._to_statement(config)
-        cache.put("builder", cache_key_str, sql_statement)
+        cache.put_builder(cache_key_str, sql_statement)
 
         return sql_statement
 
@@ -614,7 +780,8 @@ class QueryBuilder(ABC):
         Returns:
             SQL: A SQL statement object.
         """
-        safe_query = self.build()
+        dialect_override = config.dialect if config else None
+        safe_query = self.build(dialect=dialect_override)
 
         kwargs, parameters = self._extract_statement_parameters(safe_query.parameters)
 
@@ -633,7 +800,8 @@ class QueryBuilder(ABC):
             and isinstance(self._expression, exp.Expression)
         ):
             try:
-                sql_string = self._expression.sql(dialect=config.dialect, pretty=True)
+                identify = self._should_identify(config.dialect)
+                sql_string = self._expression.sql(dialect=config.dialect, pretty=True, identify=identify)
             except Exception:
                 sql_string = safe_query.sql
 
@@ -649,7 +817,7 @@ class QueryBuilder(ABC):
         """Extract parameters for SQL statement creation.
 
         Args:
-            raw_parameters: Raw parameter data from SafeQuery
+            raw_parameters: Raw parameter data from BuiltQuery
 
         Returns:
             Tuple of (kwargs, parameters) for SQL statement construction
@@ -684,7 +852,9 @@ class QueryBuilder(ABC):
             return self.dialect.__name__.lower()
         if isinstance(self.dialect, Dialect):
             return type(self.dialect).__name__.lower()
-        return getattr(self.dialect, "__name__", str(self.dialect)).lower()
+        if has_name(self.dialect):
+            return self.dialect.__name__.lower()
+        return str(self.dialect).lower()
 
     def _merge_sql_object_parameters(self, sql_obj: Any) -> None:
         """Merge parameters from a SQL object into the builder.
@@ -695,7 +865,7 @@ class QueryBuilder(ABC):
         if not has_expression_and_parameters(sql_obj):
             return
 
-        sql_parameters = getattr(sql_obj, "parameters", {})
+        sql_parameters = sql_obj.parameters
         for param_name, param_value in sql_parameters.items():
             unique_name = self._generate_unique_parameter_name(param_name)
             self.add_parameter(param_value, name=unique_name)
@@ -709,6 +879,40 @@ class QueryBuilder(ABC):
         """Set query parameters (public API)."""
         self._parameters = parameters.copy()
 
+    def _is_oracle_dialect(self, dialect: "DialectType | str | None") -> bool:
+        """Check if target dialect is Oracle."""
+        if dialect is None:
+            return False
+        return str(dialect).lower() == "oracle"
+
+    def _unquote_identifiers_for_oracle(self, expression: exp.Expression) -> exp.Expression:
+        """Remove identifier quoting to avoid Oracle case-sensitive lookup issues."""
+
+        return expression.copy().transform(_unquote_identifier, copy=False)
+
+    def _strip_lock_identifier_quotes(self, sql_string: str) -> str:
+        for keyword in ("FOR UPDATE OF ", "FOR SHARE OF "):
+            if keyword in sql_string and not self._lock_targets_quoted:
+                head, tail = sql_string.split(keyword, 1)
+                tail = tail.replace('"', "")
+                return f"{head}{keyword}{tail}"
+        if sql_string.startswith('MERGE INTO "') and not self._merge_target_quoted:
+            # Remove quotes around target table only, leave alias/rest intact
+            end_quote = sql_string.find('"', len('MERGE INTO "'))
+            if end_quote > 0:
+                table_name = sql_string[len('MERGE INTO "') : end_quote]
+                remainder = sql_string[end_quote + 1 :]
+                return f"MERGE INTO {table_name}{remainder}"
+        return sql_string
+
+    def _should_identify(self, dialect: "DialectType | str | None") -> bool:
+        """Determine whether to quote identifiers for the given dialect."""
+        if dialect is None:
+            return True
+        dialect_name = str(dialect).lower()
+        # Oracle folds unquoted identifiers to uppercase; quoting lower-case breaks table lookup
+        return dialect_name != "oracle"
+
     @property
     def with_ctes(self) -> "dict[str, exp.CTE]":
         """Get WITH clause CTEs (public API)."""
@@ -717,3 +921,102 @@ class QueryBuilder(ABC):
     def generate_unique_parameter_name(self, base_name: str) -> str:
         """Generate unique parameter name (public API)."""
         return self._generate_unique_parameter_name(base_name)
+
+    def build_static_expression(
+        self,
+        expression: exp.Expression | None = None,
+        parameters: dict[str, Any] | None = None,
+        *,
+        cache_key: str | None = None,
+        expression_factory: Callable[[], exp.Expression] | None = None,
+        copy: bool = True,
+        optimize_expression: bool | None = None,
+        dialect: DialectType | None = None,
+    ) -> "BuiltQuery":
+        """Compile a pre-built expression with optional caching and parameters.
+
+        Designed for hot paths that construct an AST once and reuse it with
+        different parameters, avoiding repeated parse/optimize cycles.
+
+        Args:
+            expression: Pre-built sqlglot expression to render (required when cache_key is not provided).
+            parameters: Optional parameter mapping to include in the result.
+            cache_key: When provided, the expression will be cached under this key.
+            expression_factory: Factory used to build the expression on cache miss.
+            copy: Copy the expression before rendering to avoid caller mutation.
+            optimize_expression: Override builder optimization toggle for this call.
+            dialect: Optional dialect override for SQL generation.
+
+        Returns:
+            BuiltQuery containing SQL and parameters.
+        """
+
+        expr: exp.Expression | None = None
+
+        if cache_key is not None:
+            cache = get_cache()
+            cached_expr = cache.get_expression(cache_key)
+            if cached_expr is None:
+                if expression_factory is None:
+                    msg = "expression_factory is required when cache_key is provided"
+                    self._raise_sql_builder_error(msg)
+                expr_candidate = expression_factory()
+                if not is_expression(expr_candidate):
+                    self._raise_invalid_expression_type(expr_candidate)
+                expr_to_store = expr_candidate.copy() if copy else expr_candidate
+                should_optimize = self.enable_optimization if optimize_expression is None else optimize_expression
+                if should_optimize:
+                    expr_to_store = self._optimize_expression(expr_to_store)
+                cache.put_expression(cache_key, expr_to_store)
+                cached_expr = expr_to_store
+            expr = cached_expr.copy() if copy else cached_expr
+        else:
+            if expression is None:
+                msg = "expression must be provided when cache_key is not set"
+                self._raise_sql_builder_error(msg)
+            expr = expression.copy() if copy else expression
+            should_optimize = self.enable_optimization if optimize_expression is None else optimize_expression
+            if should_optimize:
+                expr = self._optimize_expression(expr)
+
+        if expr is None:
+            self._raise_sql_builder_error("Static expression could not be resolved.")
+
+        target_dialect = str(dialect) if dialect else self.dialect_name
+        identify = self._should_identify(target_dialect)
+        sql_string = expr.sql(dialect=target_dialect, pretty=True, identify=identify)
+        return BuiltQuery(
+            sql=sql_string, parameters=parameters.copy() if parameters else {}, dialect=dialect or self.dialect
+        )
+
+
+class ExpressionBuilder(QueryBuilder):
+    """Builder wrapper for a pre-parsed SQLGlot expression."""
+
+    __slots__ = ()
+
+    def __init__(self, expression: exp.Expression, **kwargs: Any) -> None:
+        (dialect, schema, enable_optimization, optimize_joins, optimize_predicates, simplify_expressions) = (
+            self._parse_query_builder_kwargs(kwargs)
+        )
+        super().__init__(
+            dialect=dialect,
+            schema=schema,
+            enable_optimization=enable_optimization,
+            optimize_joins=optimize_joins,
+            optimize_predicates=optimize_predicates,
+            simplify_expressions=simplify_expressions,
+        )
+        if not is_expression(expression):
+            self._raise_invalid_expression_type(expression)
+        self._expression = expression
+
+    def _create_base_expression(self) -> exp.Expression:
+        if self._expression is None:
+            msg = "ExpressionBuilder requires an expression at construction."
+            self._raise_sql_builder_error(msg)
+        return self._expression
+
+    @property
+    def _expected_result_type(self) -> "type[SQLResult]":
+        return SQLResult
