@@ -9,7 +9,7 @@ from typing import TYPE_CHECKING, Any, Union
 from sqlglot import exp
 from typing_extensions import Self
 
-from sqlspec.builder._base import QueryBuilder, SafeQuery
+from sqlspec.builder._base import BuiltQuery, QueryBuilder
 from sqlspec.builder._select import Select
 from sqlspec.core import SQL, SQLResult
 from sqlspec.utils.type_guards import has_sqlglot_expression, has_with_method
@@ -185,7 +185,7 @@ class DDLBuilder(QueryBuilder):
     def _expected_result_type(self) -> "type[SQLResult]":
         return SQLResult
 
-    def build(self, dialect: "DialectType" = None) -> "SafeQuery":
+    def build(self, dialect: "DialectType" = None) -> "BuiltQuery":
         if self._expression is None:
             self._expression = self._create_base_expression()
         return super().build(dialect=dialect)
@@ -817,28 +817,36 @@ class CreateIndex(DDLBuilder):
         return self
 
     def _create_base_expression(self) -> exp.Expression:
+        """Build the CREATE INDEX expression used by this builder.
+
+        Columns are turned into raw expressions (not ``Ordered``) to preserve natural NULL ordering,
+        string ``where`` clauses become expressions, and the final ``exp.Index`` is wrapped in an ``exp.Create`` with the configured flags.
+        """
         if not self._index_name or not self._table_name:
             self._raise_sql_builder_error("Index name and table name must be set for CREATE INDEX.")
-        exprs: list[exp.Expression] = []
+
+        cols: list[exp.Expression] = []
         for col in self._columns:
             if isinstance(col, str):
-                exprs.append(exp.column(col))
+                cols.append(exp.column(col))
             else:
-                exprs.append(col)
+                cols.append(col)
+
         where_expr = None
         if self._where:
             where_expr = exp.condition(self._where) if isinstance(self._where, str) else self._where
-        return exp.Create(
-            kind="INDEX",
-            this=exp.to_identifier(self._index_name),
-            table=exp.to_table(self._table_name),
-            expressions=exprs,
-            unique=self._unique,
-            exists=self._if_not_exists,
-            using=exp.to_identifier(self._using) if self._using else None,
-            where=where_expr,
+
+        index_params = exp.IndexParameters(columns=cols) if cols else None
+
+        index_expr = exp.Index(
+            this=exp.to_identifier(self._index_name), table=exp.to_table(self._table_name), params=index_params
         )
 
+        if where_expr:
+            index_expr.set("where", where_expr)
+
+        return exp.Create(kind="INDEX", this=index_expr, unique=self._unique, exists=self._if_not_exists)
+
 
 class Truncate(DDLBuilder):
     """Builder for TRUNCATE TABLE ... [CASCADE|RESTRICT] [RESTART IDENTITY|CONTINUE IDENTITY]."""
@@ -1147,8 +1155,8 @@ class CreateMaterializedView(DDLBuilder):
         if self._select_query is None:
             self._raise_sql_builder_error("SELECT query must be set for CREATE MATERIALIZED VIEW.")
 
-        select_expr = None
-        select_parameters = None
+        select_expr: exp.Expression | None = None
+        select_parameters: dict[str, Any] | None = None
 
         if isinstance(self._select_query, SQL):
             select_expr = self._select_query.expression
@@ -1243,8 +1251,8 @@ class CreateView(DDLBuilder):
         if self._select_query is None:
             self._raise_sql_builder_error("SELECT query must be set for CREATE VIEW.")
 
-        select_expr = None
-        select_parameters = None
+        select_expr: exp.Expression | None = None
+        select_parameters: dict[str, Any] | None = None
 
         if isinstance(self._select_query, SQL):
             select_expr = self._select_query.expression
