@@ -3,6 +3,7 @@
 Provides statement builders (select, insert, update, etc.) and column expressions.
 """
 
+import hashlib
 import logging
 from collections.abc import Mapping, Sequence
 from typing import TYPE_CHECKING, Any, Union, cast
@@ -30,6 +31,7 @@ from sqlspec.builder._ddl import (
     Truncate,
 )
 from sqlspec.builder._delete import Delete
+from sqlspec.builder._explain import Explain
 from sqlspec.builder._expression_wrappers import (
     AggregateExpression,
     ConversionExpression,
@@ -38,18 +40,21 @@ from sqlspec.builder._expression_wrappers import (
     StringExpression,
 )
 from sqlspec.builder._insert import Insert
-from sqlspec.builder._join import JoinBuilder
+from sqlspec.builder._join import JoinBuilder, create_join_builder
 from sqlspec.builder._merge import Merge
 from sqlspec.builder._parsing_utils import extract_expression, to_expression
 from sqlspec.builder._select import Case, Select, SubqueryBuilder, WindowFunctionBuilder
 from sqlspec.builder._update import Update
 from sqlspec.core import SQL
+from sqlspec.core.explain import ExplainFormat, ExplainOptions
 from sqlspec.exceptions import SQLBuilderError
+from sqlspec.utils.logging import get_logger
 
 if TYPE_CHECKING:
     from collections.abc import Mapping, Sequence
 
     from sqlspec.builder._expression_wrappers import ExpressionWrapper
+    from sqlspec.protocols import SQLBuilderProtocol
 
 
 __all__ = (
@@ -68,6 +73,7 @@ __all__ = (
     "DropSchema",
     "DropTable",
     "DropView",
+    "Explain",
     "Insert",
     "Merge",
     "RenameTable",
@@ -82,7 +88,7 @@ __all__ = (
     "sql",
 )
 
-logger = logging.getLogger("sqlspec")
+logger = get_logger("sqlspec.builder.factory")
 
 MIN_SQL_LIKE_STRING_LENGTH = 6
 MIN_DECODE_ARGS = 2
@@ -114,6 +120,11 @@ SQL_STARTERS = {
 }
 
 
+def _fingerprint_sql(sql: str) -> str:
+    digest = hashlib.sha256(sql.encode("utf-8", errors="replace")).hexdigest()
+    return digest[:12]
+
+
 def _normalize_copy_dialect(dialect: DialectType | None) -> str:
     if dialect is None:
         return "postgres"
@@ -207,22 +218,40 @@ def build_copy_to_statement(
 class SQLFactory:
     """Factory for creating SQL builders and column expressions."""
 
+    @staticmethod
+    def _detect_type_from_expression(parsed_expr: exp.Expression) -> str:
+        if parsed_expr.key:
+            return parsed_expr.key.upper()
+        command_type = type(parsed_expr).__name__.upper()
+        if command_type == "COMMAND" and parsed_expr.this:
+            return str(parsed_expr.this).upper()
+        return command_type
+
+    @staticmethod
+    def _parse_sql_expression(sql: str, dialect: DialectType | None) -> "exp.Expression | None":
+        try:
+            return sqlglot.parse_one(sql, read=dialect)
+        except SQLGlotParseError:
+            if logger.isEnabledFor(logging.DEBUG):
+                logger.debug(
+                    "Failed to parse SQL for type detection",
+                    extra={"sql_length": len(sql), "sql_hash": _fingerprint_sql(sql)},
+                )
+        except (ValueError, TypeError, AttributeError):
+            if logger.isEnabledFor(logging.DEBUG):
+                logger.debug(
+                    "Unexpected error during SQL type detection",
+                    exc_info=True,
+                    extra={"sql_length": len(sql), "sql_hash": _fingerprint_sql(sql)},
+                )
+        return None
+
     @classmethod
     def detect_sql_type(cls, sql: str, dialect: DialectType = None) -> str:
-        try:
-            parsed_expr = sqlglot.parse_one(sql, read=dialect)
-            if parsed_expr and parsed_expr.key:
-                return parsed_expr.key.upper()
-            if parsed_expr:
-                command_type = type(parsed_expr).__name__.upper()
-                if command_type == "COMMAND" and parsed_expr.this:
-                    return str(parsed_expr.this).upper()
-                return command_type
-        except SQLGlotParseError:
-            logger.debug("Failed to parse SQL for type detection: %s", sql[:100])
-        except (ValueError, TypeError, AttributeError) as e:
-            logger.warning("Unexpected error during SQL type detection for '%s...': %s", sql[:50], e)
-        return "UNKNOWN"
+        parsed_expr = cls._parse_sql_expression(sql, dialect)
+        if parsed_expr is None:
+            return "UNKNOWN"
+        return cls._detect_type_from_expression(parsed_expr)
 
     def __init__(self, dialect: DialectType = None) -> None:
         """Initialize the SQL factory.
@@ -286,7 +315,8 @@ class SQLFactory:
         if len(columns_or_sql) == 1 and isinstance(columns_or_sql[0], str):
             sql_candidate = columns_or_sql[0].strip()
             if self._looks_like_sql(sql_candidate):
-                detected = self.detect_sql_type(sql_candidate, dialect=builder_dialect)
+                parsed_expr = self._parse_sql_expression(sql_candidate, builder_dialect)
+                detected = "UNKNOWN" if parsed_expr is None else self._detect_type_from_expression(parsed_expr)
                 if detected not in {"SELECT", "WITH"}:
                     msg = (
                         f"sql.select() expects a SELECT or WITH statement, got {detected}. "
@@ -294,7 +324,7 @@ class SQLFactory:
                     )
                     raise SQLBuilderError(msg)
                 select_builder = Select(dialect=builder_dialect)
-                return self._populate_select_from_sql(select_builder, sql_candidate)
+                return self._populate_select_from_sql(select_builder, sql_candidate, parsed_expr)
         select_builder = Select(dialect=builder_dialect)
         if columns_or_sql:
             select_builder.select(*columns_or_sql)
@@ -305,7 +335,8 @@ class SQLFactory:
         builder = Insert(dialect=builder_dialect)
         if table_or_sql:
             if self._looks_like_sql(table_or_sql):
-                detected = self.detect_sql_type(table_or_sql, dialect=builder_dialect)
+                parsed_expr = self._parse_sql_expression(table_or_sql, builder_dialect)
+                detected = "UNKNOWN" if parsed_expr is None else self._detect_type_from_expression(parsed_expr)
                 if detected not in {"INSERT", "SELECT"}:
                     msg = (
                         f"sql.insert() expects INSERT or SELECT (for insert-from-select), got {detected}. "
@@ -313,7 +344,7 @@ class SQLFactory:
                         f"or ensure the SQL is INSERT/SELECT."
                     )
                     raise SQLBuilderError(msg)
-                return self._populate_insert_from_sql(builder, table_or_sql)
+                return self._populate_insert_from_sql(builder, table_or_sql, parsed_expr)
             return builder.into(table_or_sql)
         return builder
 
@@ -322,46 +353,109 @@ class SQLFactory:
         builder = Update(dialect=builder_dialect)
         if table_or_sql:
             if self._looks_like_sql(table_or_sql):
-                detected = self.detect_sql_type(table_or_sql, dialect=builder_dialect)
+                parsed_expr = self._parse_sql_expression(table_or_sql, builder_dialect)
+                detected = "UNKNOWN" if parsed_expr is None else self._detect_type_from_expression(parsed_expr)
                 if detected != "UPDATE":
                     msg = (
                         f"sql.update() expects UPDATE statement, got {detected}. "
                         f"Use sql.{detected.lower()}() if a dedicated builder exists."
                     )
                     raise SQLBuilderError(msg)
-                return self._populate_update_from_sql(builder, table_or_sql)
+                return self._populate_update_from_sql(builder, table_or_sql, parsed_expr)
             return builder.table(table_or_sql)
         return builder
 
     def delete(self, table_or_sql: str | None = None, dialect: DialectType = None) -> "Delete":
         builder_dialect = dialect or self.dialect
-        builder = Delete(dialect=builder_dialect)
         if table_or_sql and self._looks_like_sql(table_or_sql):
-            detected = self.detect_sql_type(table_or_sql, dialect=builder_dialect)
+            builder = Delete(dialect=builder_dialect)
+            parsed_expr = self._parse_sql_expression(table_or_sql, builder_dialect)
+            detected = "UNKNOWN" if parsed_expr is None else self._detect_type_from_expression(parsed_expr)
             if detected != "DELETE":
                 msg = (
                     f"sql.delete() expects DELETE statement, got {detected}. "
                     f"Use sql.{detected.lower()}() if a dedicated builder exists."
                 )
                 raise SQLBuilderError(msg)
-            return self._populate_delete_from_sql(builder, table_or_sql)
-        return builder
+            return self._populate_delete_from_sql(builder, table_or_sql, parsed_expr)
+
+        return Delete(table_or_sql, dialect=builder_dialect) if table_or_sql else Delete(dialect=builder_dialect)
 
     def merge(self, table_or_sql: str | None = None, dialect: DialectType = None) -> "Merge":
         builder_dialect = dialect or self.dialect
-        builder = Merge(dialect=builder_dialect)
-        if table_or_sql:
-            if self._looks_like_sql(table_or_sql):
-                detected = self.detect_sql_type(table_or_sql, dialect=builder_dialect)
-                if detected != "MERGE":
-                    msg = (
-                        f"sql.merge() expects MERGE statement, got {detected}. "
-                        f"Use sql.{detected.lower()}() if a dedicated builder exists."
-                    )
-                    raise SQLBuilderError(msg)
-                return self._populate_merge_from_sql(builder, table_or_sql)
-            return builder.into(table_or_sql)
-        return builder
+        if table_or_sql and self._looks_like_sql(table_or_sql):
+            builder = Merge(dialect=builder_dialect)
+            parsed_expr = self._parse_sql_expression(table_or_sql, builder_dialect)
+            detected = "UNKNOWN" if parsed_expr is None else self._detect_type_from_expression(parsed_expr)
+            if detected != "MERGE":
+                msg = (
+                    f"sql.merge() expects MERGE statement, got {detected}. "
+                    f"Use sql.{detected.lower()}() if a dedicated builder exists."
+                )
+                raise SQLBuilderError(msg)
+            return self._populate_merge_from_sql(builder, table_or_sql, parsed_expr)
+
+        return Merge(table_or_sql, dialect=builder_dialect) if table_or_sql else Merge(dialect=builder_dialect)
+
+    def explain(
+        self,
+        statement: "str | exp.Expression | SQL | SQLBuilderProtocol",
+        *,
+        analyze: bool = False,
+        verbose: bool = False,
+        format: "ExplainFormat | str | None" = None,
+        dialect: DialectType = None,
+    ) -> "Explain":
+        """Create an EXPLAIN builder for a SQL statement.
+
+        Wraps any SQL statement in an EXPLAIN clause with dialect-aware
+        syntax generation.
+
+        Args:
+            statement: SQL statement to explain (string, expression, SQL object, or builder)
+            analyze: Execute the statement and show actual runtime statistics
+            verbose: Show additional information
+            format: Output format (TEXT, JSON, XML, YAML, TREE, TRADITIONAL)
+            dialect: Optional SQL dialect override
+
+        Returns:
+            Explain builder for further configuration
+
+        Examples:
+            Basic EXPLAIN:
+                plan = sql.explain("SELECT * FROM users").build()
+
+            With options:
+                plan = (
+                    sql.explain("SELECT * FROM users", analyze=True, format="json")
+                    .buffers()
+                    .timing()
+                    .build()
+                )
+
+            From QueryBuilder:
+                query = sql.select("*").from_("users").where("id = :id", id=1)
+                plan = sql.explain(query, analyze=True).build()
+
+            Chained configuration:
+                plan = (
+                    sql.explain(sql.select("*").from_("large_table"))
+                    .analyze()
+                    .format("json")
+                    .buffers()
+                    .timing()
+                    .build()
+                )
+        """
+        builder_dialect = dialect or self.dialect
+
+        fmt = None
+        if format is not None:
+            fmt = ExplainFormat(format.lower()) if isinstance(format, str) else format
+
+        options = ExplainOptions(analyze=analyze, verbose=verbose, format=fmt)
+
+        return Explain(statement, dialect=builder_dialect, options=options)
 
     @property
     def merge_(self) -> "Merge":
@@ -649,83 +743,140 @@ class SQLFactory:
 
         return False
 
-    def _populate_insert_from_sql(self, builder: "Insert", sql_string: str) -> "Insert":
+    def _populate_insert_from_sql(
+        self, builder: "Insert", sql_string: str, parsed_expr: "exp.Expression | None" = None
+    ) -> "Insert":
         """Parse SQL string and populate INSERT builder using SQLGlot directly."""
         try:
-            parsed_expr: exp.Expression = exp.maybe_parse(sql_string, dialect=self.dialect)
+            if parsed_expr is None:
+                parsed_expr = exp.maybe_parse(sql_string, dialect=self.dialect)
 
             if isinstance(parsed_expr, exp.Insert):
                 builder.set_expression(parsed_expr)
                 return builder
 
             if isinstance(parsed_expr, exp.Select):
-                logger.info("Detected SELECT statement for INSERT - may need target table specification")
+                logger.debug(
+                    "Detected SELECT statement for INSERT; builder requires explicit target table",
+                    extra={"builder": "insert"},
+                )
                 return builder
 
-            logger.warning("Cannot create INSERT from %s statement", type(parsed_expr).__name__)
+            logger.debug(
+                "Cannot create INSERT from parsed statement type",
+                extra={"builder": "insert", "parsed_type": type(parsed_expr).__name__},
+            )
 
-        except Exception as e:
-            logger.warning("Failed to parse INSERT SQL, falling back to traditional mode: %s", e)
+        except Exception:
+            logger.debug(
+                "Failed to parse INSERT SQL; falling back to traditional mode",
+                exc_info=True,
+                extra={"builder": "insert"},
+            )
         return builder
 
-    def _populate_select_from_sql(self, builder: "Select", sql_string: str) -> "Select":
+    def _populate_select_from_sql(
+        self, builder: "Select", sql_string: str, parsed_expr: "exp.Expression | None" = None
+    ) -> "Select":
         """Parse SQL string and populate SELECT builder using SQLGlot directly."""
         try:
-            parsed_expr: exp.Expression = exp.maybe_parse(sql_string, dialect=self.dialect)
+            if parsed_expr is None:
+                parsed_expr = exp.maybe_parse(sql_string, dialect=self.dialect)
 
+            if isinstance(parsed_expr, exp.With):
+                base_expression = parsed_expr.this
+                if isinstance(base_expression, exp.Select):
+                    builder.set_expression(base_expression)
+                    builder.load_ctes(list(parsed_expr.expressions))
+                    return builder
             if isinstance(parsed_expr, exp.Select):
                 builder.set_expression(parsed_expr)
                 return builder
 
-            logger.warning("Cannot create SELECT from %s statement", type(parsed_expr).__name__)
+            logger.debug(
+                "Cannot create SELECT from parsed statement type",
+                extra={"builder": "select", "parsed_type": type(parsed_expr).__name__},
+            )
 
-        except Exception as e:
-            logger.warning("Failed to parse SELECT SQL, falling back to traditional mode: %s", e)
+        except Exception:
+            logger.debug(
+                "Failed to parse SELECT SQL; falling back to traditional mode",
+                exc_info=True,
+                extra={"builder": "select"},
+            )
         return builder
 
-    def _populate_update_from_sql(self, builder: "Update", sql_string: str) -> "Update":
+    def _populate_update_from_sql(
+        self, builder: "Update", sql_string: str, parsed_expr: "exp.Expression | None" = None
+    ) -> "Update":
         """Parse SQL string and populate UPDATE builder using SQLGlot directly."""
         try:
-            parsed_expr: exp.Expression = exp.maybe_parse(sql_string, dialect=self.dialect)
+            if parsed_expr is None:
+                parsed_expr = exp.maybe_parse(sql_string, dialect=self.dialect)
 
             if isinstance(parsed_expr, exp.Update):
                 builder.set_expression(parsed_expr)
                 return builder
 
-            logger.warning("Cannot create UPDATE from %s statement", type(parsed_expr).__name__)
+            logger.debug(
+                "Cannot create UPDATE from parsed statement type",
+                extra={"builder": "update", "parsed_type": type(parsed_expr).__name__},
+            )
 
-        except Exception as e:
-            logger.warning("Failed to parse UPDATE SQL, falling back to traditional mode: %s", e)
+        except Exception:
+            logger.debug(
+                "Failed to parse UPDATE SQL; falling back to traditional mode",
+                exc_info=True,
+                extra={"builder": "update"},
+            )
         return builder
 
-    def _populate_delete_from_sql(self, builder: "Delete", sql_string: str) -> "Delete":
+    def _populate_delete_from_sql(
+        self, builder: "Delete", sql_string: str, parsed_expr: "exp.Expression | None" = None
+    ) -> "Delete":
         """Parse SQL string and populate DELETE builder using SQLGlot directly."""
         try:
-            parsed_expr: exp.Expression = exp.maybe_parse(sql_string, dialect=self.dialect)
+            if parsed_expr is None:
+                parsed_expr = exp.maybe_parse(sql_string, dialect=self.dialect)
 
             if isinstance(parsed_expr, exp.Delete):
                 builder.set_expression(parsed_expr)
                 return builder
 
-            logger.warning("Cannot create DELETE from %s statement", type(parsed_expr).__name__)
+            logger.debug(
+                "Cannot create DELETE from parsed statement type",
+                extra={"builder": "delete", "parsed_type": type(parsed_expr).__name__},
+            )
 
-        except Exception as e:
-            logger.warning("Failed to parse DELETE SQL, falling back to traditional mode: %s", e)
+        except Exception:
+            logger.debug(
+                "Failed to parse DELETE SQL; falling back to traditional mode",
+                exc_info=True,
+                extra={"builder": "delete"},
+            )
         return builder
 
-    def _populate_merge_from_sql(self, builder: "Merge", sql_string: str) -> "Merge":
+    def _populate_merge_from_sql(
+        self, builder: "Merge", sql_string: str, parsed_expr: "exp.Expression | None" = None
+    ) -> "Merge":
         """Parse SQL string and populate MERGE builder using SQLGlot directly."""
         try:
-            parsed_expr: exp.Expression = exp.maybe_parse(sql_string, dialect=self.dialect)
+            if parsed_expr is None:
+                parsed_expr = exp.maybe_parse(sql_string, dialect=self.dialect)
 
             if isinstance(parsed_expr, exp.Merge):
                 builder.set_expression(parsed_expr)
                 return builder
 
-            logger.warning("Cannot create MERGE from %s statement", type(parsed_expr).__name__)
+            logger.debug(
+                "Cannot create MERGE from parsed statement type",
+                extra={"builder": "merge", "parsed_type": type(parsed_expr).__name__},
+            )
 
-        except Exception as e:
-            logger.warning("Failed to parse MERGE SQL, falling back to traditional mode: %s", e)
+        except Exception:
+            logger.debug(
+                "Failed to parse MERGE SQL; falling back to traditional mode", exc_info=True, extra={"builder": "merge"}
+            )
         return builder
 
     def column(self, name: str, table: str | None = None) -> Column:
@@ -750,13 +901,15 @@ class SQLFactory:
         Example:
             ```python
             case_expr = (
-                sql.case_.when("x = 1", "one")
+                sql.case_
+                .when("x = 1", "one")
                 .when("x = 2", "two")
                 .else_("other")
                 .end()
             )
             aliased_case = (
-                sql.case_.when("status = 'active'", 1)
+                sql.case_
+                .when("status = 'active'", 1)
                 .else_(0)
                 .as_("is_active")
             )
@@ -812,27 +965,27 @@ class SQLFactory:
     @property
     def inner_join_(self) -> "JoinBuilder":
         """Create an INNER JOIN builder."""
-        return JoinBuilder("inner join")
+        return create_join_builder("inner join")
 
     @property
     def left_join_(self) -> "JoinBuilder":
         """Create a LEFT JOIN builder."""
-        return JoinBuilder("left join")
+        return create_join_builder("left join")
 
     @property
     def right_join_(self) -> "JoinBuilder":
         """Create a RIGHT JOIN builder."""
-        return JoinBuilder("right join")
+        return create_join_builder("right join")
 
     @property
     def full_join_(self) -> "JoinBuilder":
         """Create a FULL OUTER JOIN builder."""
-        return JoinBuilder("full join")
+        return create_join_builder("full join")
 
     @property
     def cross_join_(self) -> "JoinBuilder":
         """Create a CROSS JOIN builder."""
-        return JoinBuilder("cross join")
+        return create_join_builder("cross join")
 
     @property
     def lateral_join_(self) -> "JoinBuilder":
@@ -844,13 +997,14 @@ class SQLFactory:
         Example:
             ```python
             query = (
-                sql.select("u.name", "arr.value")
+                sql
+                .select("u.name", "arr.value")
                 .from_("users u")
                 .join(sql.lateral_join_("UNNEST(u.tags)").on("true"))
             )
             ```
         """
-        return JoinBuilder("lateral join", lateral=True)
+        return create_join_builder("lateral join", lateral=True)
 
     @property
     def left_lateral_join_(self) -> "JoinBuilder":
@@ -859,7 +1013,7 @@ class SQLFactory:
         Returns:
             JoinBuilder configured for LEFT LATERAL JOIN
         """
-        return JoinBuilder("left join", lateral=True)
+        return create_join_builder("left join", lateral=True)
 
     @property
     def cross_lateral_join_(self) -> "JoinBuilder":
@@ -868,7 +1022,7 @@ class SQLFactory:
         Returns:
             JoinBuilder configured for CROSS LATERAL JOIN
         """
-        return JoinBuilder("cross join", lateral=True)
+        return create_join_builder("cross join", lateral=True)
 
     def __getattr__(self, name: str) -> "Column":
         """Dynamically create column references.
@@ -1033,7 +1187,8 @@ class SQLFactory:
         Example:
             ```python
             query = (
-                sql.select("product", "region", sql.sum("sales"))
+                sql
+                .select("product", "region", sql.sum("sales"))
                 .from_("sales_data")
                 .group_by(sql.rollup("product", "region"))
             )
@@ -1055,7 +1210,8 @@ class SQLFactory:
         Example:
             ```python
             query = (
-                sql.select("product", "region", sql.sum("sales"))
+                sql
+                .select("product", "region", sql.sum("sales"))
                 .from_("sales_data")
                 .group_by(sql.cube("product", "region"))
             )
@@ -1077,7 +1233,8 @@ class SQLFactory:
         Example:
             ```python
             query = (
-                sql.select("product", "region", sql.sum("sales"))
+                sql
+                .select("product", "region", sql.sum("sales"))
                 .from_("sales_data")
                 .group_by(
                     sql.grouping_sets(("product",), ("region",), ())
@@ -1112,7 +1269,8 @@ class SQLFactory:
             ```python
             subquery = sql.select("user_id").from_("active_users")
             query = (
-                sql.select("*")
+                sql
+                .select("*")
                 .from_("users")
                 .where(sql.id.eq(sql.any(subquery)))
             )
@@ -1140,7 +1298,8 @@ class SQLFactory:
             ```python
             subquery = sql.select("user_id").from_("blocked_users")
             query = (
-                sql.select("*")
+                sql
+                .select("*")
                 .from_("users")
                 .where(sql.id.neq(sql.not_any(subquery)))
             )
@@ -1426,7 +1585,8 @@ class SQLFactory:
 
 
             truncate_sql = (
-                sql.truncate_table("my_table")
+                sql
+                .truncate_table("my_table")
                 .cascade()
                 .restart_identity()
                 .build()
