@@ -8,9 +8,12 @@ from typing import TYPE_CHECKING, Any
 
 from sqlglot import exp
 
-from sqlspec.utils.type_guards import is_typed_parameter
+from sqlspec.core.parameters import TypedParameter
+from sqlspec.utils.type_guards import is_expression, is_typed_parameter
 
 if TYPE_CHECKING:
+    from collections.abc import Sequence
+
     from sqlspec.core.filters import StatementFilter
     from sqlspec.core.statement import SQL
 
@@ -23,7 +26,7 @@ __all__ = (
 )
 
 
-def hash_expression(expr: exp.Expression | None, _seen: set[int] | None = None) -> int:
+def hash_expression(expr: "exp.Expression | None", _seen: "set[int] | None" = None) -> int:
     """Generate hash from AST structure.
 
     Args:
@@ -53,7 +56,7 @@ def hash_expression(expr: exp.Expression | None, _seen: set[int] | None = None)
     return hash(tuple(components))
 
 
-def _hash_value(value: Any, _seen: set[int]) -> int:
+def _hash_value(value: Any, _seen: "set[int]") -> int:
     """Hash different value types.
 
     Args:
@@ -77,8 +80,8 @@ def _hash_value(value: Any, _seen: set[int]) -> int:
 
 
 def hash_parameters(
-    positional_parameters: list[Any] | None = None,
-    named_parameters: dict[str, Any] | None = None,
+    positional_parameters: "list[Any] | None" = None,
+    named_parameters: "dict[str, Any] | None" = None,
     original_parameters: Any | None = None,
 ) -> int:
     """Generate hash for SQL parameters.
@@ -94,8 +97,6 @@ def hash_parameters(
     param_hash = 0
 
     if positional_parameters:
-        from sqlspec.core.parameters import TypedParameter
-
         hashable_parameters: list[tuple[Any, Any]] = []
         for param in positional_parameters:
             if isinstance(param, TypedParameter):
@@ -115,7 +116,7 @@ def hash_parameters(
         param_hash ^= hash(tuple(hashable_parameters))
 
     if named_parameters:
-        hashable_items = []
+        hashable_items: list[tuple[str, tuple[Any, Any]]] = []
         for key, value in sorted(named_parameters.items()):
             if is_typed_parameter(value):
                 if isinstance(value.value, (list, dict)):
@@ -148,7 +149,7 @@ def _hash_filter_value(value: Any) -> int:
         return hash(repr(value))
 
 
-def hash_filters(filters: list["StatementFilter"] | None = None) -> int:
+def hash_filters(filters: "Sequence[StatementFilter] | None" = None) -> int:
     """Generate hash for statement filters.
 
     Args:
@@ -160,18 +161,7 @@ def hash_filters(filters: list["StatementFilter"] | None = None) -> int:
     if not filters:
         return 0
 
-    filter_components = []
-    for f in filters:
-        components: list[Any] = [f.__class__.__name__]
-
-        filter_dict = getattr(f, "__dict__", None)
-        if filter_dict is not None:
-            for key, value in sorted(filter_dict.items()):
-                components.append((key, _hash_filter_value(value)))
-
-        filter_components.append(tuple(components))
-
-    return hash(tuple(filter_components))
+    return hash(tuple(f.get_cache_key() for f in filters))
 
 
 def hash_sql_statement(statement: "SQL") -> str:
@@ -183,8 +173,6 @@ def hash_sql_statement(statement: "SQL") -> str:
     Returns:
         Cache key string
     """
-    from sqlspec.utils.type_guards import is_expression
-
     stmt_expr = statement.statement_expression
     expr_hash = hash_expression(stmt_expr) if is_expression(stmt_expr) else hash(statement.raw_sql)
 
@@ -235,8 +223,8 @@ def hash_expression_node(node: exp.Expression, include_children: bool = True, di
 def hash_optimized_expression(
     expr: exp.Expression,
     dialect: str,
-    schema: dict[str, Any] | None = None,
-    optimizer_settings: dict[str, Any] | None = None,
+    schema: "dict[str, Any] | None" = None,
+    optimizer_settings: "dict[str, Any] | None" = None,
 ) -> str:
     """Generate cache key for optimized expressions.
 
