@@ -10,6 +10,7 @@ __all__ = (
     "DataError",
     "DatabaseConnectionError",
     "DialectNotSupportedError",
+    "EventChannelError",
     "FileNotFoundInStorageError",
     "ForeignKeyViolationError",
     "ImproperConfigurationError",
@@ -29,11 +30,13 @@ __all__ = (
     "SQLFileParseError",
     "SQLParsingError",
     "SQLSpecError",
+    "SerializationConflictError",
     "SerializationError",
     "StackExecutionError",
     "StorageCapabilityError",
     "StorageOperationFailedError",
     "TransactionError",
+    "TransactionRetryError",
     "UniqueViolationError",
 )
 
@@ -41,7 +44,7 @@ __all__ = (
 class SQLSpecError(Exception):
     """Base exception class for SQLSpec exceptions."""
 
-    detail: str
+    detail: str = ""
 
     def __init__(self, *args: Any, detail: str = "") -> None:
         """Initialize SQLSpecError.
@@ -52,11 +55,10 @@ class SQLSpecError(Exception):
         """
         str_args = [str(arg) for arg in args if arg]
         if not detail:
-            if str_args:
-                detail, *str_args = str_args
-            elif hasattr(self, "detail"):
-                detail = self.detail
+            detail = str_args[0] if str_args else self.detail
         self.detail = detail
+        if detail and detail not in str_args:
+            str_args = [detail, *str_args]
         super().__init__(*str_args)
 
     def __repr__(self) -> str:
@@ -65,7 +67,10 @@ class SQLSpecError(Exception):
         return self.__class__.__name__
 
     def __str__(self) -> str:
-        return " ".join((*self.args, self.detail)).strip()
+        parts = list(self.args)
+        if self.detail and self.detail not in self.args:
+            parts.append(self.detail)
+        return " ".join(parts).strip()
 
 
 class MissingDependencyError(SQLSpecError, ImportError):
@@ -86,6 +91,10 @@ class BackendNotRegisteredError(SQLSpecError):
         super().__init__(f"Storage backend '{backend_key}' is not registered. Please register it before use.")
 
 
+class EventChannelError(SQLSpecError):
+    """Raised when event channel operations fail."""
+
+
 class ConfigResolverError(SQLSpecError):
     """Exception raised when config resolution fails."""
 
@@ -169,6 +178,14 @@ class TransactionError(SQLSpecError):
     """Transaction error (rollback, deadlock, serialization failure)."""
 
 
+class SerializationConflictError(TransactionError):
+    """Serialization conflict (SQLSTATE 40001) requiring retry."""
+
+
+class TransactionRetryError(TransactionError):
+    """Transaction failed after retries were exhausted."""
+
+
 class DataError(SQLSpecError):
     """Invalid data type or format for database operation."""
 
