@@ -1,3 +1,4 @@
+import logging
 from collections.abc import Iterable
 from contextlib import suppress
 from dataclasses import dataclass, field
@@ -18,6 +19,7 @@ from sqlspec.config import (
     SyncConfigT,
     SyncDatabaseConfig,
 )
+from sqlspec.core.filters import OffsetPagination
 from sqlspec.exceptions import ImproperConfigurationError
 from sqlspec.extensions.litestar._utils import (
     delete_sqlspec_scope_state,
@@ -34,7 +36,7 @@ from sqlspec.extensions.litestar.handlers import (
 )
 from sqlspec.typing import NUMPY_INSTALLED, ConnectionT, PoolT, SchemaT
 from sqlspec.utils.correlation import CorrelationContext
-from sqlspec.utils.logging import get_logger
+from sqlspec.utils.logging import get_logger, log_with_context
 from sqlspec.utils.serializers import numpy_array_dec_hook, numpy_array_enc_hook, numpy_array_predicate
 
 if TYPE_CHECKING:
@@ -50,7 +52,7 @@ if TYPE_CHECKING:
     from sqlspec.driver import AsyncDriverAdapterBase, SyncDriverAdapterBase
     from sqlspec.loader import SQLFileLoader
 
-logger = get_logger("extensions.litestar")
+logger = get_logger("sqlspec.extensions.litestar")
 
 CommitMode = Literal["manual", "autocommit", "autocommit_include_redirect"]
 DEFAULT_COMMIT_MODE: CommitMode = "manual"
@@ -71,15 +73,24 @@ TRACE_CONTEXT_FALLBACK_HEADERS: tuple[str, ...] = (
 CORRELATION_STATE_KEY = "sqlspec_correlation_id"
 
 __all__ = (
+    "CORRELATION_STATE_KEY",
     "DEFAULT_COMMIT_MODE",
     "DEFAULT_CONNECTION_KEY",
+    "DEFAULT_CORRELATION_HEADER",
     "DEFAULT_POOL_KEY",
     "DEFAULT_SESSION_KEY",
+    "TRACE_CONTEXT_FALLBACK_HEADERS",
     "CommitMode",
+    "CorrelationMiddleware",
+    "PluginConfigState",
     "SQLSpecPlugin",
 )
 
 
+def _encode_offset_pagination(value: OffsetPagination[Any]) -> dict[str, Any]:
+    return {"items": value.items, "limit": value.limit, "offset": value.offset, "total": value.total}
+
+
 def _normalize_header_list(headers: Any) -> list[str]:
     if headers is None:
         return []
@@ -117,7 +128,7 @@ def _build_correlation_headers(*, primary: str, configured: list[str], auto_trac
     return tuple(_dedupe_headers(header_order))
 
 
-class _CorrelationMiddleware:
+class CorrelationMiddleware:
     __slots__ = ("_app", "_headers")
 
     def __init__(self, app: "ASGIApp", *, headers: tuple[str, ...]) -> None:
@@ -142,6 +153,7 @@ class _CorrelationMiddleware:
         if not header_value:
             header_value = CorrelationContext.generate()
 
+        previous_correlation_id = CorrelationContext.get()
         CorrelationContext.set(header_value)
         set_sqlspec_scope_state(scope, CORRELATION_STATE_KEY, header_value)
         try:
@@ -149,11 +161,11 @@ class _CorrelationMiddleware:
         finally:
             with suppress(KeyError):
                 delete_sqlspec_scope_state(scope, CORRELATION_STATE_KEY)
-            CorrelationContext.clear()
+            CorrelationContext.set(previous_correlation_id)
 
 
 @dataclass
-class _PluginConfigState:
+class PluginConfigState:
     """Internal state for each database configuration."""
 
     config: "DatabaseConfigProtocol[Any, Any, Any]"
@@ -189,7 +201,7 @@ class SQLSpecPlugin(InitPluginProtocol, CLIPlugin):
 
     Example:
         config = AsyncpgConfig(
-            pool_config={"dsn": "postgresql://localhost/db"},
+            connection_config={"dsn": "postgresql://localhost/db"},
             extension_config={
                 "litestar": {
                     "session_table": "custom_sessions"  # Optional custom table name
@@ -219,7 +231,7 @@ class SQLSpecPlugin(InitPluginProtocol, CLIPlugin):
         """
         self._sqlspec = sqlspec
 
-        self._plugin_configs: list[_PluginConfigState] = []
+        self._plugin_configs: list[PluginConfigState] = []
         for cfg in self._sqlspec.configs.values():
             config_union = cast(
                 "SyncDatabaseConfig[Any, Any, Any] | NoPoolSyncConfig[Any, Any] | AsyncDatabaseConfig[Any, Any, Any] | NoPoolAsyncConfig[Any, Any]",
@@ -237,6 +249,15 @@ class SQLSpecPlugin(InitPluginProtocol, CLIPlugin):
                 if header not in correlation_headers:
                     correlation_headers.append(header)
         self._correlation_headers = tuple(correlation_headers)
+        log_with_context(
+            logger,
+            logging.DEBUG,
+            "extension.init",
+            framework="litestar",
+            stage="init",
+            config_count=len(self._plugin_configs),
+            correlation_headers=len(self._correlation_headers),
+        )
 
     def _extract_litestar_settings(
         self,
@@ -276,9 +297,9 @@ class SQLSpecPlugin(InitPluginProtocol, CLIPlugin):
         self,
         config: "SyncDatabaseConfig[Any, Any, Any] | NoPoolSyncConfig[Any, Any] | AsyncDatabaseConfig[Any, Any, Any] | NoPoolAsyncConfig[Any, Any]",
         settings: "dict[str, Any]",
-    ) -> _PluginConfigState:
+    ) -> PluginConfigState:
         """Create plugin state with handlers for the given configuration."""
-        state = _PluginConfigState(
+        state = PluginConfigState(
             config=config,
             connection_key=settings["connection_key"],
             pool_key=settings["pool_key"],
@@ -296,7 +317,7 @@ class SQLSpecPlugin(InitPluginProtocol, CLIPlugin):
             self._setup_handlers(state)
         return state
 
-    def _setup_handlers(self, state: _PluginConfigState) -> None:
+    def _setup_handlers(self, state: PluginConfigState) -> None:
         """Setup handlers for the plugin state."""
         connection_key = state.connection_key
         pool_key = state.pool_key
@@ -385,6 +406,13 @@ class SQLSpecPlugin(InitPluginProtocol, CLIPlugin):
         if signature_namespace:
             app_config.signature_namespace.update(signature_namespace)
 
+        if app_config.type_encoders is None:
+            app_config.type_encoders = {OffsetPagination: _encode_offset_pagination}
+        else:
+            encoders_dict = dict(app_config.type_encoders)
+            encoders_dict[OffsetPagination] = _encode_offset_pagination
+            app_config.type_encoders = encoders_dict
+
         if NUMPY_INSTALLED:
             import numpy as np
 
@@ -403,11 +431,21 @@ class SQLSpecPlugin(InitPluginProtocol, CLIPlugin):
                 app_config.type_decoders = decoders_list
 
         if self._correlation_headers:
-            middleware = DefineMiddleware(_CorrelationMiddleware, headers=self._correlation_headers)
+            middleware = DefineMiddleware(CorrelationMiddleware, headers=self._correlation_headers)
             existing_middleware = list(app_config.middleware or [])
             existing_middleware.append(middleware)
             app_config.middleware = existing_middleware
 
+        log_with_context(
+            logger,
+            logging.DEBUG,
+            "extension.init",
+            framework="litestar",
+            stage="configured",
+            config_count=len(self._plugin_configs),
+            correlation_headers=len(self._correlation_headers),
+            numpy_enabled=bool(NUMPY_INSTALLED),
+        )
         return app_config
 
     def get_annotations(
@@ -579,7 +617,7 @@ class SQLSpecPlugin(InitPluginProtocol, CLIPlugin):
 
     def _get_plugin_state(
         self, key: "str | SyncConfigT | AsyncConfigT | type[SyncConfigT | AsyncConfigT]"
-    ) -> _PluginConfigState:
+    ) -> PluginConfigState:
         """Get plugin state for a configuration by key."""
         if isinstance(key, str):
             for state in self._plugin_configs:
