@@ -5,6 +5,7 @@ This module provides the main command interface for database migrations.
 
 import functools
 import inspect
+import logging
 import time
 from collections.abc import Awaitable, Callable
 from typing import TYPE_CHECKING, Any, ParamSpec, TypeVar, cast
@@ -19,8 +20,9 @@ from sqlspec.migrations.fix import MigrationFixer
 from sqlspec.migrations.runner import AsyncMigrationRunner, SyncMigrationRunner
 from sqlspec.migrations.utils import create_migration_file
 from sqlspec.migrations.validation import validate_migration_order
-from sqlspec.utils.logging import get_logger
-from sqlspec.utils.version import generate_conversion_map, generate_timestamp_version
+from sqlspec.migrations.version import generate_conversion_map, generate_timestamp_version, parse_version
+from sqlspec.observability import resolve_db_system
+from sqlspec.utils.logging import get_logger, log_with_context
 
 if TYPE_CHECKING:
     from pathlib import Path
@@ -29,7 +31,7 @@ if TYPE_CHECKING:
 
 __all__ = ("AsyncMigrationCommands", "SyncMigrationCommands", "create_migration_commands")
 
-logger = get_logger("migrations.commands")
+logger = get_logger("sqlspec.migrations.commands")
 console = Console()
 P = ParamSpec("P")
 R = TypeVar("R")
@@ -56,7 +58,7 @@ def _with_command_span(
         signature = inspect.signature(func)
 
         def _prepare(self: Any, args: tuple[Any, ...], kwargs: dict[str, Any]) -> tuple[Any, bool, Any]:
-            runtime = getattr(self, "_runtime", None)
+            runtime = self._runtime
             metadata_args = _bind_arguments(signature, args, kwargs)
             dry_run = False
             if dry_run_param is not None:
@@ -82,10 +84,10 @@ def _with_command_span(
             recorded_error: bool,
             dry_run: bool,
         ) -> None:
-            command_error = getattr(self, "_last_command_error", None)
-            setattr(self, "_last_command_error", None)
-            command_metrics = getattr(self, "_last_command_metrics", None)
-            setattr(self, "_last_command_metrics", None)
+            command_error = self._last_command_error
+            self._last_command_error = None
+            command_metrics = self._last_command_metrics
+            self._last_command_metrics = None
             if runtime is None:
                 return
             if command_error is not None and not recorded_error:
@@ -202,11 +204,22 @@ class SyncMigrationCommands(BaseMigrationCommands["SyncConfigT", Any]):
 
             current = self.tracker.get_current_version(driver)
             if not current:
+                log_with_context(
+                    logger,
+                    logging.DEBUG,
+                    "migration.list",
+                    db_system=resolve_db_system(type(driver).__name__),
+                    current_version=None,
+                    applied_count=0,
+                    verbose=verbose,
+                    status="empty",
+                )
                 console.print("[yellow]No migrations applied yet[/]")
                 return None
 
             console.print(f"[green]Current version:[/] {current}")
 
+            applied: list[dict[str, Any]] = []
             if verbose:
                 applied = self.tracker.get_applied_migrations(driver)
 
@@ -228,6 +241,17 @@ class SyncMigrationCommands(BaseMigrationCommands["SyncConfigT", Any]):
 
                 console.print(table)
 
+            applied_count = len(applied) if verbose else None
+            log_with_context(
+                logger,
+                logging.DEBUG,
+                "migration.list",
+                db_system=resolve_db_system(type(driver).__name__),
+                current_version=current,
+                applied_count=applied_count,
+                verbose=verbose,
+                status="complete",
+            )
             return cast("str | None", current)
 
     def _load_single_migration_checksum(self, version: str, file_path: "Path") -> "tuple[str, tuple[str, Path]] | None":
@@ -243,8 +267,18 @@ class SyncMigrationCommands(BaseMigrationCommands["SyncConfigT", Any]):
         try:
             migration = self.runner.load_migration(file_path, version)
             return (version, (migration["checksum"], file_path))
-        except Exception as e:
-            logger.debug("Could not load migration %s for auto-sync: %s", version, e)
+        except Exception as exc:
+            log_with_context(
+                logger,
+                logging.DEBUG,
+                "migration.list",
+                db_system=resolve_db_system(type(self.config).__name__),
+                version=version,
+                file_path=str(file_path),
+                error_type=type(exc).__name__,
+                status="failed",
+                operation="load_checksum",
+            )
             return None
 
     def _load_migration_checksums(self, all_migrations: "list[tuple[str, Path]]") -> "dict[str, tuple[str, Path]]":
@@ -282,8 +316,16 @@ class SyncMigrationCommands(BaseMigrationCommands["SyncConfigT", Any]):
 
         try:
             applied_migrations = self.tracker.get_applied_migrations(driver)
-        except Exception:
-            logger.debug("Could not fetch applied migrations for synchronization (table schema may be migrating)")
+        except Exception as exc:
+            log_with_context(
+                logger,
+                logging.DEBUG,
+                "migration.list",
+                db_system=resolve_db_system(type(driver).__name__),
+                error_type=type(exc).__name__,
+                status="failed",
+                operation="applied_fetch",
+            )
             return 0
 
         applied_map = {m["version_num"]: m for m in applied_migrations}
@@ -369,8 +411,6 @@ class SyncMigrationCommands(BaseMigrationCommands["SyncConfigT", Any]):
                     if revision == "head":
                         pending.append((version, file_path))
                     else:
-                        from sqlspec.utils.version import parse_version
-
                         parsed_version = parse_version(version)
                         parsed_revision = parse_version(revision)
                         if parsed_version <= parsed_revision:
@@ -389,7 +429,7 @@ class SyncMigrationCommands(BaseMigrationCommands["SyncConfigT", Any]):
                 return
             pending_versions = [v for v, _ in pending]
 
-            migration_config = getattr(self.config, "migration_config", {}) or {}
+            migration_config = cast("dict[str, Any]", self.config.migration_config) or {}
             strict_ordering = migration_config.get("strict_ordering", False) and not allow_missing
 
             validate_migration_order(pending_versions, applied_versions, strict_ordering)
@@ -458,8 +498,6 @@ class SyncMigrationCommands(BaseMigrationCommands["SyncConfigT", Any]):
             elif revision == "base":
                 to_revert = list(reversed(applied))
             else:
-                from sqlspec.utils.version import parse_version
-
                 parsed_revision = parse_version(revision)
                 for migration in reversed(applied):
                     parsed_migration_version = parse_version(migration["version_num"])
@@ -548,6 +586,16 @@ class SyncMigrationCommands(BaseMigrationCommands["SyncConfigT", Any]):
             config=self.config,
             template_settings=self._template_settings,
         )
+        log_with_context(
+            logger,
+            logging.DEBUG,
+            "migration.create",
+            db_system=resolve_db_system(type(self.config).__name__),
+            version=version,
+            file_path=str(file_path),
+            file_type=selected_format,
+            description=message,
+        )
         console.print(f"[green]Created migration:[/] {file_path}")
 
     def fix(self, dry_run: bool = False, update_database: bool = True, yes: bool = False) -> None:
@@ -630,7 +678,6 @@ class SyncMigrationCommands(BaseMigrationCommands["SyncConfigT", Any]):
             console.print("[green]✓ Conversion complete![/]")
 
         except Exception as e:
-            logger.exception("Fix command failed")
             console.print(f"[red]✗ Error: {e}[/]")
             fixer.rollback()
             console.print("[yellow]Restored files from backup[/]")
@@ -685,10 +732,21 @@ class AsyncMigrationCommands(BaseMigrationCommands["AsyncConfigT", Any]):
 
             current = await self.tracker.get_current_version(driver)
             if not current:
+                log_with_context(
+                    logger,
+                    logging.DEBUG,
+                    "migration.list",
+                    db_system=resolve_db_system(type(driver).__name__),
+                    current_version=None,
+                    applied_count=0,
+                    verbose=verbose,
+                    status="empty",
+                )
                 console.print("[yellow]No migrations applied yet[/]")
                 return None
 
             console.print(f"[green]Current version:[/] {current}")
+            applied: list[dict[str, Any]] = []
             if verbose:
                 applied = await self.tracker.get_applied_migrations(driver)
                 table = Table(title="Applied Migrations")
@@ -707,6 +765,17 @@ class AsyncMigrationCommands(BaseMigrationCommands["AsyncConfigT", Any]):
                     )
                 console.print(table)
 
+            applied_count = len(applied) if verbose else None
+            log_with_context(
+                logger,
+                logging.DEBUG,
+                "migration.list",
+                db_system=resolve_db_system(type(driver).__name__),
+                current_version=current,
+                applied_count=applied_count,
+                verbose=verbose,
+                status="complete",
+            )
             return cast("str | None", current)
 
     async def _load_single_migration_checksum(
@@ -724,8 +793,18 @@ class AsyncMigrationCommands(BaseMigrationCommands["AsyncConfigT", Any]):
         try:
             migration = await self.runner.load_migration(file_path, version)
             return (version, (migration["checksum"], file_path))
-        except Exception as e:
-            logger.debug("Could not load migration %s for auto-sync: %s", version, e)
+        except Exception as exc:
+            log_with_context(
+                logger,
+                logging.DEBUG,
+                "migration.list",
+                db_system=resolve_db_system(type(self.config).__name__),
+                version=version,
+                file_path=str(file_path),
+                error_type=type(exc).__name__,
+                status="failed",
+                operation="load_checksum",
+            )
             return None
 
     async def _load_migration_checksums(
@@ -765,8 +844,16 @@ class AsyncMigrationCommands(BaseMigrationCommands["AsyncConfigT", Any]):
 
         try:
             applied_migrations = await self.tracker.get_applied_migrations(driver)
-        except Exception:
-            logger.debug("Could not fetch applied migrations for synchronization (table schema may be migrating)")
+        except Exception as exc:
+            log_with_context(
+                logger,
+                logging.DEBUG,
+                "migration.list",
+                db_system=resolve_db_system(type(driver).__name__),
+                error_type=type(exc).__name__,
+                status="failed",
+                operation="applied_fetch",
+            )
             return 0
 
         applied_map = {m["version_num"]: m for m in applied_migrations}
@@ -834,7 +921,7 @@ class AsyncMigrationCommands(BaseMigrationCommands["AsyncConfigT", Any]):
             await self.tracker.ensure_tracking_table(driver)
 
             if auto_sync:
-                migration_config = getattr(self.config, "migration_config", {}) or {}
+                migration_config = cast("dict[str, Any]", self.config.migration_config) or {}
                 config_auto_sync = migration_config.get("auto_sync", True)
                 if config_auto_sync:
                     await self._synchronize_version_records(driver)
@@ -853,8 +940,6 @@ class AsyncMigrationCommands(BaseMigrationCommands["AsyncConfigT", Any]):
                     if revision == "head":
                         pending.append((version, file_path))
                     else:
-                        from sqlspec.utils.version import parse_version
-
                         parsed_version = parse_version(version)
                         parsed_revision = parse_version(revision)
                         if parsed_version <= parsed_revision:
@@ -873,7 +958,7 @@ class AsyncMigrationCommands(BaseMigrationCommands["AsyncConfigT", Any]):
                 return
             pending_versions = [v for v, _ in pending]
 
-            migration_config = getattr(self.config, "migration_config", {}) or {}
+            migration_config = cast("dict[str, Any]", self.config.migration_config) or {}
             strict_ordering = migration_config.get("strict_ordering", False) and not allow_missing
 
             validate_migration_order(pending_versions, applied_versions, strict_ordering)
@@ -940,8 +1025,6 @@ class AsyncMigrationCommands(BaseMigrationCommands["AsyncConfigT", Any]):
             elif revision == "base":
                 to_revert = list(reversed(applied))
             else:
-                from sqlspec.utils.version import parse_version
-
                 parsed_revision = parse_version(revision)
                 for migration in reversed(applied):
                     parsed_migration_version = parse_version(migration["version_num"])
@@ -1035,6 +1118,16 @@ class AsyncMigrationCommands(BaseMigrationCommands["AsyncConfigT", Any]):
             config=self.config,
             template_settings=self._template_settings,
         )
+        log_with_context(
+            logger,
+            logging.DEBUG,
+            "migration.create",
+            db_system=resolve_db_system(type(self.config).__name__),
+            version=version,
+            file_path=str(file_path),
+            file_type=selected_format,
+            description=message,
+        )
         console.print(f"[green]Created migration:[/] {file_path}")
 
     async def fix(self, dry_run: bool = False, update_database: bool = True, yes: bool = False) -> None:
@@ -1117,7 +1210,6 @@ class AsyncMigrationCommands(BaseMigrationCommands["AsyncConfigT", Any]):
             console.print("[green]✓ Conversion complete![/]")
 
         except Exception as e:
-            logger.exception("Fix command failed")
             console.print(f"[red]✗ Error: {e}[/]")
             fixer.rollback()
             console.print("[yellow]Restored files from backup[/]")
