@@ -8,7 +8,85 @@ from mypy_extensions import mypyc_attr
 
 from sqlspec.typing import ArrowRecordBatch, ArrowTable
 
-__all__ = ("ObjectStoreBase",)
+__all__ = ("AsyncArrowBatchIterator", "AsyncBytesIterator", "ObjectStoreBase")
+
+
+class AsyncArrowBatchIterator:
+    """Async iterator wrapper for sync Arrow batch iterators.
+
+    This class implements the async iterator protocol without using async generators,
+    allowing it to be compiled by mypyc (which doesn't support async generators).
+
+    The class wraps a synchronous iterator and exposes it as an async iterator,
+    enabling usage with `async for` syntax.
+    """
+
+    __slots__ = ("_sync_iter",)
+
+    def __init__(self, sync_iterator: "Iterator[ArrowRecordBatch]") -> None:
+        """Initialize the async iterator wrapper.
+
+        Args:
+            sync_iterator: The synchronous iterator to wrap.
+        """
+        self._sync_iter = sync_iterator
+
+    def __aiter__(self) -> "AsyncArrowBatchIterator":
+        """Return self as the async iterator."""
+        return self
+
+    async def __anext__(self) -> "ArrowRecordBatch":
+        """Get the next item from the iterator asynchronously.
+
+        Returns:
+            The next Arrow record batch.
+
+        Raises:
+            StopAsyncIteration: When the iterator is exhausted.
+        """
+        try:
+            return next(self._sync_iter)
+        except StopIteration:
+            raise StopAsyncIteration from None
+
+
+class AsyncBytesIterator:
+    """Async iterator wrapper for sync bytes iterators.
+
+    This class implements the async iterator protocol without using async generators,
+    allowing it to be compiled by mypyc (which doesn't support async generators).
+
+    The class wraps a synchronous iterator and exposes it as an async iterator,
+    enabling usage with `async for` syntax.
+    """
+
+    __slots__ = ("_sync_iter",)
+
+    def __init__(self, sync_iterator: "Iterator[bytes]") -> None:
+        """Initialize the async iterator wrapper.
+
+        Args:
+            sync_iterator: The synchronous iterator to wrap.
+        """
+        self._sync_iter = sync_iterator
+
+    def __aiter__(self) -> "AsyncBytesIterator":
+        """Return self as the async iterator."""
+        return self
+
+    async def __anext__(self) -> bytes:
+        """Get the next item from the iterator asynchronously.
+
+        Returns:
+            The next chunk of bytes.
+
+        Raises:
+            StopAsyncIteration: When the iterator is exhausted.
+        """
+        try:
+            return next(self._sync_iter)
+        except StopIteration:
+            raise StopAsyncIteration from None
 
 
 @mypyc_attr(allow_interpreted_subclasses=True)
@@ -27,6 +105,11 @@ class ObjectStoreBase(ABC):
         """Write bytes to storage."""
         raise NotImplementedError
 
+    @abstractmethod
+    def stream_read(self, path: str, chunk_size: "int | None" = None, **kwargs: Any) -> Iterator[bytes]:
+        """Stream bytes from storage."""
+        raise NotImplementedError
+
     @abstractmethod
     def read_text(self, path: str, encoding: str = "utf-8", **kwargs: Any) -> str:
         """Read text from storage."""
@@ -38,7 +121,7 @@ class ObjectStoreBase(ABC):
         raise NotImplementedError
 
     @abstractmethod
-    def list_objects(self, prefix: str = "", recursive: bool = True, **kwargs: Any) -> list[str]:
+    def list_objects(self, prefix: str = "", recursive: bool = True, **kwargs: Any) -> "list[str]":
         """List objects in storage."""
         raise NotImplementedError
 
@@ -63,12 +146,12 @@ class ObjectStoreBase(ABC):
         raise NotImplementedError
 
     @abstractmethod
-    def glob(self, pattern: str, **kwargs: Any) -> list[str]:
+    def glob(self, pattern: str, **kwargs: Any) -> "list[str]":
         """Find objects matching pattern."""
         raise NotImplementedError
 
     @abstractmethod
-    def get_metadata(self, path: str, **kwargs: Any) -> dict[str, Any]:
+    def get_metadata(self, path: str, **kwargs: Any) -> "dict[str, object]":
         """Get object metadata from storage."""
         raise NotImplementedError
 
@@ -118,7 +201,14 @@ class ObjectStoreBase(ABC):
         raise NotImplementedError
 
     @abstractmethod
-    async def list_objects_async(self, prefix: str = "", recursive: bool = True, **kwargs: Any) -> list[str]:
+    async def stream_read_async(
+        self, path: str, chunk_size: "int | None" = None, **kwargs: Any
+    ) -> AsyncIterator[bytes]:
+        """Stream bytes from storage asynchronously."""
+        raise NotImplementedError
+
+    @abstractmethod
+    def list_objects_async(self, prefix: str = "", recursive: bool = True, **kwargs: Any) -> "list[str]":
         """List objects in storage asynchronously."""
         raise NotImplementedError
 
@@ -143,7 +233,7 @@ class ObjectStoreBase(ABC):
         raise NotImplementedError
 
     @abstractmethod
-    async def get_metadata_async(self, path: str, **kwargs: Any) -> dict[str, Any]:
+    def get_metadata_async(self, path: str, **kwargs: Any) -> "dict[str, object]":
         """Get object metadata from storage asynchronously."""
         raise NotImplementedError
 
