@@ -11,7 +11,7 @@ import re
 from collections.abc import AsyncIterator, Iterator
 from functools import partial
 from pathlib import Path, PurePosixPath
-from typing import Any, Final, cast
+from typing import Any, Final, cast, overload
 from urllib.parse import urlparse
 
 from mypy_extensions import mypyc_attr
@@ -20,78 +20,51 @@ from sqlspec.exceptions import StorageOperationFailedError
 from sqlspec.storage._utils import import_pyarrow, import_pyarrow_parquet, resolve_storage_path
 from sqlspec.storage.errors import execute_sync_storage_operation
 from sqlspec.typing import ArrowRecordBatch, ArrowTable
+from sqlspec.utils.logging import get_logger, log_with_context
 from sqlspec.utils.module_loader import ensure_obstore
-from sqlspec.utils.sync_tools import async_
 
 __all__ = ("ObStoreBackend",)
 
-logger = logging.getLogger(__name__)
-
-
-class _AsyncArrowIterator:
-    """Helper class to work around mypyc's lack of async generator support.
-
-    Uses hybrid async/sync pattern:
-    - Native async I/O for network operations (S3, GCS, Azure)
-    - Thread pool for CPU-bound PyArrow parsing
-    """
-
-    __slots__ = ("_current_file_iterator", "_files_iterator", "backend", "kwargs", "pattern")
-
-    def __init__(self, backend: "ObStoreBackend", pattern: str, **kwargs: Any) -> None:
-        self.backend = backend
-        self.pattern = pattern
-        self.kwargs = kwargs
-        self._files_iterator: Iterator[str] | None = None
-        self._current_file_iterator: Iterator[ArrowRecordBatch] | None = None
-
-    def __aiter__(self) -> "_AsyncArrowIterator":
-        return self
-
-    async def __anext__(self) -> ArrowRecordBatch:
-        pq = import_pyarrow_parquet()
-
-        if self._files_iterator is None:
-            files = self.backend.glob(self.pattern, **self.kwargs)
-            self._files_iterator = iter(files)
-
-        while True:
-            if self._current_file_iterator is not None:
-
-                def _safe_next_batch() -> ArrowRecordBatch:
-                    try:
-                        return next(self._current_file_iterator)  # type: ignore[arg-type]
-                    except StopIteration as e:
-                        raise StopAsyncIteration from e
-
-                try:
-                    return await async_(_safe_next_batch)()
-                except StopAsyncIteration:
-                    self._current_file_iterator = None
-                    continue
-
-            try:
-                next_file = next(self._files_iterator)
-            except StopIteration as e:
-                raise StopAsyncIteration from e
-
-            data = await self.backend.read_bytes_async(next_file)
-            parquet_file = pq.ParquetFile(io.BytesIO(data))
-            self._current_file_iterator = parquet_file.iter_batches()
-
-    async def aclose(self) -> None:
-        """Close underlying file iterator."""
-        if self._current_file_iterator is not None:
-            try:
-                close_method = self._current_file_iterator.close  # type: ignore[attr-defined]
-                await async_(close_method)()  # pyright: ignore
-            except AttributeError:
-                pass
+logger = get_logger(__name__)
 
 
 DEFAULT_OPTIONS: Final[dict[str, Any]] = {"connect_timeout": "30s", "request_timeout": "60s"}
 
 
+def _log_storage_event(
+    event: str,
+    *,
+    backend_type: str,
+    protocol: str,
+    operation: str | None = None,
+    mode: str | None = "sync",
+    path: str | None = None,
+    source_path: str | None = None,
+    destination_path: str | None = None,
+    count: int | None = None,
+    exists: bool | None = None,
+) -> None:
+    fields: dict[str, Any] = {
+        "backend_type": backend_type,
+        "protocol": protocol,
+        "mode": mode,
+        "path": path,
+        "source_path": source_path,
+        "destination_path": destination_path,
+        "count": count,
+        "exists": exists,
+    }
+    if operation is not None:
+        fields["operation"] = operation
+    log_with_context(logger, logging.DEBUG, event, **fields)
+
+
+def _read_obstore_bytes(store: Any, resolved_path: str) -> bytes:
+    """Read bytes via obstore."""
+    result = store.get(resolved_path)
+    return cast("bytes", result.bytes().to_bytes())
+
+
 @mypyc_attr(allow_interpreted_subclasses=True)
 class ObStoreBackend:
     """Object storage backend using obstore.
@@ -160,14 +133,25 @@ class ObStoreBackend:
 
                 self.store = from_url(uri, **kwargs)  # pyright: ignore[reportAttributeAccessIssue]
 
-            logger.debug("ObStore backend initialized for %s", uri)
+            _log_storage_event(
+                "storage.backend.ready",
+                backend_type=self.backend_type,
+                protocol=self.protocol,
+                operation="init",
+                path=uri,
+            )
 
         except Exception as exc:
             msg = f"Failed to initialize obstore backend for {uri}"
             raise StorageOperationFailedError(msg) from exc
 
+    @property
+    def is_local_store(self) -> bool:
+        """Return whether the backend uses local storage."""
+        return self._is_local_store
+
     @classmethod
-    def from_config(cls, config: dict[str, Any]) -> "ObStoreBackend":
+    def from_config(cls, config: "dict[str, Any]") -> "ObStoreBackend":
         """Create backend from configuration dictionary."""
         store_uri = config["store_uri"]
         base_path = config.get("base_path", "")
@@ -179,6 +163,11 @@ class ObStoreBackend:
 
         return cls(uri=store_uri, **kwargs)
 
+    def _resolve_path(self, path: "str | Path") -> str:
+        if self._is_local_store:
+            return self._resolve_path_for_local_store(path)
+        return resolve_storage_path(path, self.base_path, self.protocol, strip_file_scheme=True)
+
     def _resolve_path_for_local_store(self, path: "str | Path") -> str:
         """Resolve path for LocalStore which expects relative paths from its root."""
 
@@ -194,32 +183,40 @@ class ObStoreBackend:
 
     def read_bytes(self, path: "str | Path", **kwargs: Any) -> bytes:  # pyright: ignore[reportUnusedParameter]
         """Read bytes using obstore."""
-        if self._is_local_store:
-            resolved_path = self._resolve_path_for_local_store(path)
-        else:
-            resolved_path = resolve_storage_path(path, self.base_path, self.protocol, strip_file_scheme=True)
+        resolved_path = self._resolve_path(path)
 
-        def _action() -> bytes:
-            result = self.store.get(resolved_path)
-            return cast("bytes", result.bytes().to_bytes())
-
-        return execute_sync_storage_operation(
-            _action, backend=self.backend_type, operation="read_bytes", path=resolved_path
+        result = execute_sync_storage_operation(
+            partial(_read_obstore_bytes, self.store, resolved_path),
+            backend=self.backend_type,
+            operation="read_bytes",
+            path=resolved_path,
         )
+        _log_storage_event(
+            "storage.read",
+            backend_type=self.backend_type,
+            protocol=self.protocol,
+            operation="read_bytes",
+            path=resolved_path,
+        )
+        return result
 
     def write_bytes(self, path: "str | Path", data: bytes, **kwargs: Any) -> None:  # pyright: ignore[reportUnusedParameter]
         """Write bytes using obstore."""
-        if self._is_local_store:
-            resolved_path = self._resolve_path_for_local_store(path)
-        else:
-            resolved_path = resolve_storage_path(path, self.base_path, self.protocol, strip_file_scheme=True)
+        resolved_path = self._resolve_path(path)
 
         execute_sync_storage_operation(
-            lambda: self.store.put(resolved_path, data),
+            partial(self.store.put, resolved_path, data),
             backend=self.backend_type,
             operation="write_bytes",
             path=resolved_path,
         )
+        _log_storage_event(
+            "storage.write",
+            backend_type=self.backend_type,
+            protocol=self.protocol,
+            operation="write_bytes",
+            path=resolved_path,
+        )
 
     def read_text(self, path: "str | Path", encoding: str = "utf-8", **kwargs: Any) -> str:
         """Read text using obstore."""
@@ -229,7 +226,7 @@ class ObStoreBackend:
         """Write text using obstore."""
         self.write_bytes(path, data.encode(encoding), **kwargs)
 
-    def list_objects(self, prefix: str = "", recursive: bool = True, **kwargs: Any) -> list[str]:  # pyright: ignore[reportUnusedParameter]
+    def list_objects(self, prefix: str = "", recursive: bool = True, **kwargs: Any) -> "list[str]":  # pyright: ignore[reportUnusedParameter]
         """List objects using obstore."""
         resolved_prefix = (
             resolve_storage_path(prefix, self.base_path, self.protocol, strip_file_scheme=True)
@@ -237,50 +234,95 @@ class ObStoreBackend:
             else self.base_path or ""
         )
         items = self.store.list_with_delimiter(resolved_prefix) if not recursive else self.store.list(resolved_prefix)
-        paths: list[str] = []
-        for batch in items:
-            paths.extend(item["path"] for item in batch)
-        return sorted(paths)
+        paths = sorted(item["path"] for batch in items for item in batch)
+        _log_storage_event(
+            "storage.list",
+            backend_type=self.backend_type,
+            protocol=self.protocol,
+            operation="list_objects",
+            path=resolved_prefix,
+            count=len(paths),
+        )
+        return paths
 
     def exists(self, path: "str | Path", **kwargs: Any) -> bool:  # pyright: ignore[reportUnusedParameter]
         """Check if object exists using obstore."""
         try:
-            resolved_path = resolve_storage_path(path, self.base_path, self.protocol, strip_file_scheme=True)
-            self.store.head(resolved_path)
+            resolved_path = self._resolve_path(path)
+            self.store.head(resolved_path)  # pyright: ignore[reportUnknownMemberType]
         except Exception:
+            _log_storage_event(
+                "storage.read",
+                backend_type=self.backend_type,
+                protocol=self.protocol,
+                operation="exists",
+                path=str(path),
+                exists=False,
+            )
             return False
+        _log_storage_event(
+            "storage.read",
+            backend_type=self.backend_type,
+            protocol=self.protocol,
+            operation="exists",
+            path=resolved_path,
+            exists=True,
+        )
         return True
 
     def delete(self, path: "str | Path", **kwargs: Any) -> None:  # pyright: ignore[reportUnusedParameter]
         """Delete object using obstore."""
-        resolved_path = resolve_storage_path(path, self.base_path, self.protocol, strip_file_scheme=True)
+        resolved_path = self._resolve_path(path)
         execute_sync_storage_operation(
-            lambda: self.store.delete(resolved_path), backend=self.backend_type, operation="delete", path=resolved_path
+            partial(self.store.delete, resolved_path), backend=self.backend_type, operation="delete", path=resolved_path
+        )
+        _log_storage_event(
+            "storage.write",
+            backend_type=self.backend_type,
+            protocol=self.protocol,
+            operation="delete",
+            path=resolved_path,
         )
 
     def copy(self, source: "str | Path", destination: "str | Path", **kwargs: Any) -> None:  # pyright: ignore[reportUnusedParameter]
         """Copy object using obstore."""
-        source_path = resolve_storage_path(source, self.base_path, self.protocol, strip_file_scheme=True)
-        dest_path = resolve_storage_path(destination, self.base_path, self.protocol, strip_file_scheme=True)
+        source_path = self._resolve_path(source)
+        dest_path = self._resolve_path(destination)
         execute_sync_storage_operation(
-            lambda: self.store.copy(source_path, dest_path),
+            partial(self.store.copy, source_path, dest_path),
             backend=self.backend_type,
             operation="copy",
             path=f"{source_path}->{dest_path}",
         )
+        _log_storage_event(
+            "storage.write",
+            backend_type=self.backend_type,
+            protocol=self.protocol,
+            operation="copy",
+            source_path=source_path,
+            destination_path=dest_path,
+        )
 
     def move(self, source: "str | Path", destination: "str | Path", **kwargs: Any) -> None:  # pyright: ignore[reportUnusedParameter]
         """Move object using obstore."""
-        source_path = resolve_storage_path(source, self.base_path, self.protocol, strip_file_scheme=True)
-        dest_path = resolve_storage_path(destination, self.base_path, self.protocol, strip_file_scheme=True)
+        source_path = self._resolve_path(source)
+        dest_path = self._resolve_path(destination)
         execute_sync_storage_operation(
-            lambda: self.store.rename(source_path, dest_path),
+            partial(self.store.rename, source_path, dest_path),
             backend=self.backend_type,
             operation="move",
             path=f"{source_path}->{dest_path}",
         )
+        _log_storage_event(
+            "storage.write",
+            backend_type=self.backend_type,
+            protocol=self.protocol,
+            operation="move",
+            source_path=source_path,
+            destination_path=dest_path,
+        )
 
-    def glob(self, pattern: str, **kwargs: Any) -> list[str]:
+    def glob(self, pattern: str, **kwargs: Any) -> "list[str]":
         """Find objects matching pattern.
 
         Lists all objects and filters them client-side using the pattern.
@@ -304,11 +346,20 @@ class ObStoreBackend:
                     obj_path = PurePosixPath(obj)
                     if obj_path.match(resolved_pattern):
                         matching_objects.append(obj)
+            results = matching_objects
+        else:
+            results = [obj for obj in all_objects if fnmatch.fnmatch(obj, resolved_pattern)]
+        _log_storage_event(
+            "storage.list",
+            backend_type=self.backend_type,
+            protocol=self.protocol,
+            operation="glob",
+            path=resolved_pattern,
+            count=len(results),
+        )
+        return results
 
-            return matching_objects
-        return [obj for obj in all_objects if fnmatch.fnmatch(obj, resolved_pattern)]
-
-    def get_metadata(self, path: "str | Path", **kwargs: Any) -> dict[str, Any]:  # pyright: ignore[reportUnusedParameter]
+    def get_metadata(self, path: "str | Path", **kwargs: Any) -> "dict[str, object]":  # pyright: ignore[reportUnusedParameter]
         """Get object metadata using obstore."""
         resolved_path = resolve_storage_path(path, self.base_path, self.protocol, strip_file_scheme=True)
 
@@ -367,15 +418,23 @@ class ObStoreBackend:
         pq = import_pyarrow_parquet()
         resolved_path = resolve_storage_path(path, self.base_path, self.protocol, strip_file_scheme=True)
         data = self.read_bytes(resolved_path)
-        return cast(
+        result = cast(
             "ArrowTable",
             execute_sync_storage_operation(
-                lambda: pq.read_table(io.BytesIO(data), **kwargs),
+                partial(pq.read_table, io.BytesIO(data), **kwargs),
                 backend=self.backend_type,
                 operation="read_arrow",
                 path=resolved_path,
             ),
         )
+        _log_storage_event(
+            "storage.read",
+            backend_type=self.backend_type,
+            protocol=self.protocol,
+            operation="read_arrow",
+            path=resolved_path,
+        )
+        return result
 
     def write_arrow(self, path: "str | Path", table: ArrowTable, **kwargs: Any) -> None:
         """Write Arrow table using obstore."""
@@ -400,13 +459,37 @@ class ObStoreBackend:
 
         buffer = io.BytesIO()
         execute_sync_storage_operation(
-            lambda: pq.write_table(table, buffer, **kwargs),
+            partial(pq.write_table, table, buffer, **kwargs),
             backend=self.backend_type,
             operation="write_arrow",
             path=resolved_path,
         )
         buffer.seek(0)
         self.write_bytes(resolved_path, buffer.read())
+        _log_storage_event(
+            "storage.write",
+            backend_type=self.backend_type,
+            protocol=self.protocol,
+            operation="write_arrow",
+            path=resolved_path,
+        )
+
+    def stream_read(self, path: "str | Path", chunk_size: "int | None" = None, **kwargs: Any) -> Iterator[bytes]:
+        """Stream bytes using obstore.
+
+        Note:
+            For remote backends, this currently performs a full read and yields chunks
+            as obstore's sync client doesn't expose a streaming iterator.
+            Use stream_read_async for true streaming.
+        """
+        resolved_path = self._resolve_path(path)
+        data = self.read_bytes(resolved_path)
+
+        if chunk_size:
+            for i in range(0, len(data), chunk_size):
+                yield data[i : i + chunk_size]
+        else:
+            yield data
 
     def stream_arrow(self, pattern: str, **kwargs: Any) -> Iterator[ArrowRecordBatch]:
         """Stream Arrow record batches.
@@ -431,10 +514,80 @@ class ObStoreBackend:
             )
             yield from parquet_file.iter_batches()
 
-    def sign(self, path: str, expires_in: int = 3600, for_upload: bool = False) -> str:
-        """Generate a signed URL for the object."""
-        resolved_path = resolve_storage_path(path, self.base_path, self.protocol, strip_file_scheme=True)
-        return f"{self.store_uri}/{resolved_path}"
+    @property
+    def supports_signing(self) -> bool:
+        """Whether this backend supports URL signing.
+
+        Only S3, GCS, and Azure backends support pre-signed URLs.
+        Local file storage does not support URL signing.
+
+        Returns:
+            True if the protocol supports signing, False otherwise.
+        """
+        signable_protocols = {"s3", "gs", "gcs", "az", "azure"}
+        return self.protocol in signable_protocols
+
+    @overload
+    def sign_sync(self, paths: str, expires_in: int = 3600, for_upload: bool = False) -> str: ...
+
+    @overload
+    def sign_sync(self, paths: "list[str]", expires_in: int = 3600, for_upload: bool = False) -> "list[str]": ...
+
+    def sign_sync(
+        self, paths: "str | list[str]", expires_in: int = 3600, for_upload: bool = False
+    ) -> "str | list[str]":
+        """Generate signed URL(s) for the object(s).
+
+        Args:
+            paths: Single object path or list of paths to sign.
+            expires_in: URL expiration time in seconds (default: 3600, max: 604800 = 7 days).
+            for_upload: Whether the URL is for upload (PUT) vs download (GET).
+
+        Returns:
+            Single signed URL string if paths is a string, or list of signed URLs
+            if paths is a list. Preserves input type for convenience.
+
+        Raises:
+            NotImplementedError: If the backend protocol does not support signing.
+            ValueError: If expires_in exceeds maximum (604800 seconds).
+        """
+        import obstore as obs
+
+        signable_protocols = {"s3", "gs", "gcs", "az", "azure"}
+        if self.protocol not in signable_protocols:
+            msg = (
+                f"URL signing is not supported for protocol '{self.protocol}'. "
+                f"Only S3, GCS, and Azure backends support pre-signed URLs."
+            )
+            raise NotImplementedError(msg)
+
+        max_expires = 604800  # 7 days max per obstore/object_store limits
+        if expires_in > max_expires:
+            msg = f"expires_in cannot exceed {max_expires} seconds (7 days), got {expires_in}"
+            raise ValueError(msg)
+
+        from datetime import timedelta
+
+        method = "PUT" if for_upload else "GET"
+        expires_delta = timedelta(seconds=expires_in)
+
+        if isinstance(paths, str):
+            path_list = [paths]
+            is_single = True
+        else:
+            path_list = list(paths)
+            is_single = False
+
+        resolved_paths = [
+            resolve_storage_path(p, self.base_path, self.protocol, strip_file_scheme=True) for p in path_list
+        ]
+
+        try:
+            signed_urls: list[str] = obs.sign(self.store, method, resolved_paths, expires_delta)  # type: ignore[call-overload]
+            return signed_urls[0] if is_single else signed_urls
+        except Exception as exc:
+            msg = f"Failed to generate signed URL(s) for {resolved_paths}"
+            raise StorageOperationFailedError(msg) from exc
 
     async def read_bytes_async(self, path: "str | Path", **kwargs: Any) -> bytes:  # pyright: ignore[reportUnusedParameter]
         """Read bytes from storage asynchronously."""
@@ -444,8 +597,17 @@ class ObStoreBackend:
             resolved_path = resolve_storage_path(path, self.base_path, self.protocol, strip_file_scheme=True)
 
         result = await self.store.get_async(resolved_path)
-        bytes_obj = await result.bytes_async()
-        return bytes_obj.to_bytes()  # type: ignore[no-any-return]  # pyright: ignore[reportAttributeAccessIssue]
+        bytes_obj = await result.bytes_async()  # pyright: ignore[reportAttributeAccessIssue]
+        data = cast("bytes", bytes_obj.to_bytes())
+        _log_storage_event(
+            "storage.read",
+            backend_type=self.backend_type,
+            protocol=self.protocol,
+            operation="read_bytes",
+            mode="async",
+            path=resolved_path,
+        )
+        return data
 
     async def write_bytes_async(self, path: "str | Path", data: bytes, **kwargs: Any) -> None:  # pyright: ignore[reportUnusedParameter]
         """Write bytes to storage asynchronously."""
@@ -455,8 +617,43 @@ class ObStoreBackend:
             resolved_path = resolve_storage_path(path, self.base_path, self.protocol, strip_file_scheme=True)
 
         await self.store.put_async(resolved_path, data)
+        _log_storage_event(
+            "storage.write",
+            backend_type=self.backend_type,
+            protocol=self.protocol,
+            operation="write_bytes",
+            mode="async",
+            path=resolved_path,
+        )
 
-    async def list_objects_async(self, prefix: str = "", recursive: bool = True, **kwargs: Any) -> list[str]:  # pyright: ignore[reportUnusedParameter]
+    async def stream_read_async(
+        self, path: "str | Path", chunk_size: "int | None" = None, **kwargs: Any
+    ) -> AsyncIterator[bytes]:
+        """Stream bytes from storage asynchronously."""
+        if self._is_local_store:
+            resolved_path = self._resolve_path_for_local_store(path)
+        else:
+            resolved_path = resolve_storage_path(path, self.base_path, self.protocol, strip_file_scheme=True)
+
+        result = await self.store.get_async(resolved_path)
+        stream = result.stream()
+
+        async def _generator() -> AsyncIterator[bytes]:
+            async for chunk in stream:
+                yield bytes(chunk)
+
+            _log_storage_event(
+                "storage.read",
+                backend_type=self.backend_type,
+                protocol=self.protocol,
+                operation="stream_read",
+                mode="async",
+                path=resolved_path,
+            )
+
+        return _generator()
+
+    async def list_objects_async(self, prefix: str = "", recursive: bool = True, **kwargs: Any) -> "list[str]":  # pyright: ignore[reportUnusedParameter]
         """List objects in storage asynchronously."""
         resolved_prefix = (
             resolve_storage_path(prefix, self.base_path, self.protocol, strip_file_scheme=True)
@@ -472,7 +669,17 @@ class ObStoreBackend:
             base_depth = resolved_prefix.count("/")
             objects = [obj for obj in objects if obj.count("/") <= base_depth + 1]
 
-        return sorted(objects)
+        results = sorted(objects)
+        _log_storage_event(
+            "storage.list",
+            backend_type=self.backend_type,
+            protocol=self.protocol,
+            operation="list_objects",
+            mode="async",
+            path=resolved_prefix,
+            count=len(results),
+        )
+        return results
 
     async def read_text_async(self, path: "str | Path", encoding: str = "utf-8", **kwargs: Any) -> str:
         """Read text from storage asynchronously."""
@@ -494,7 +701,25 @@ class ObStoreBackend:
         try:
             await self.store.head_async(resolved_path)
         except Exception:
+            _log_storage_event(
+                "storage.read",
+                backend_type=self.backend_type,
+                protocol=self.protocol,
+                operation="exists",
+                mode="async",
+                path=str(path),
+                exists=False,
+            )
             return False
+        _log_storage_event(
+            "storage.read",
+            backend_type=self.backend_type,
+            protocol=self.protocol,
+            operation="exists",
+            mode="async",
+            path=resolved_path,
+            exists=True,
+        )
         return True
 
     async def delete_async(self, path: "str | Path", **kwargs: Any) -> None:  # pyright: ignore[reportUnusedParameter]
@@ -505,6 +730,14 @@ class ObStoreBackend:
             resolved_path = resolve_storage_path(path, self.base_path, self.protocol, strip_file_scheme=True)
 
         await self.store.delete_async(resolved_path)
+        _log_storage_event(
+            "storage.write",
+            backend_type=self.backend_type,
+            protocol=self.protocol,
+            operation="delete",
+            mode="async",
+            path=resolved_path,
+        )
 
     async def copy_async(self, source: "str | Path", destination: "str | Path", **kwargs: Any) -> None:  # pyright: ignore[reportUnusedParameter]
         """Copy object in storage asynchronously."""
@@ -516,6 +749,15 @@ class ObStoreBackend:
             dest_path = resolve_storage_path(destination, self.base_path, self.protocol, strip_file_scheme=True)
 
         await self.store.copy_async(source_path, dest_path)
+        _log_storage_event(
+            "storage.write",
+            backend_type=self.backend_type,
+            protocol=self.protocol,
+            operation="copy",
+            mode="async",
+            source_path=source_path,
+            destination_path=dest_path,
+        )
 
     async def move_async(self, source: "str | Path", destination: "str | Path", **kwargs: Any) -> None:  # pyright: ignore[reportUnusedParameter]
         """Move object in storage asynchronously."""
@@ -527,15 +769,24 @@ class ObStoreBackend:
             dest_path = resolve_storage_path(destination, self.base_path, self.protocol, strip_file_scheme=True)
 
         await self.store.rename_async(source_path, dest_path)
+        _log_storage_event(
+            "storage.write",
+            backend_type=self.backend_type,
+            protocol=self.protocol,
+            operation="move",
+            mode="async",
+            source_path=source_path,
+            destination_path=dest_path,
+        )
 
-    async def get_metadata_async(self, path: "str | Path", **kwargs: Any) -> dict[str, Any]:  # pyright: ignore[reportUnusedParameter]
+    async def get_metadata_async(self, path: "str | Path", **kwargs: Any) -> "dict[str, object]":  # pyright: ignore[reportUnusedParameter]
         """Get object metadata from storage asynchronously."""
         if self._is_local_store:
             resolved_path = self._resolve_path_for_local_store(path)
         else:
             resolved_path = resolve_storage_path(path, self.base_path, self.protocol, strip_file_scheme=True)
 
-        result: dict[str, Any] = {}
+        result: dict[str, object] = {}
         try:
             metadata = await self.store.head_async(resolved_path)
             result.update({
@@ -559,7 +810,16 @@ class ObStoreBackend:
         pq = import_pyarrow_parquet()
         resolved_path = resolve_storage_path(path, self.base_path, self.protocol, strip_file_scheme=True)
         data = await self.read_bytes_async(resolved_path)
-        return cast("ArrowTable", pq.read_table(io.BytesIO(data), **kwargs))
+        result = cast("ArrowTable", pq.read_table(io.BytesIO(data), **kwargs))
+        _log_storage_event(
+            "storage.read",
+            backend_type=self.backend_type,
+            protocol=self.protocol,
+            operation="read_arrow",
+            mode="async",
+            path=resolved_path,
+        )
+        return result
 
     async def write_arrow_async(self, path: "str | Path", table: ArrowTable, **kwargs: Any) -> None:
         """Write Arrow table to storage asynchronously."""
@@ -569,12 +829,88 @@ class ObStoreBackend:
         pq.write_table(table, buffer, **kwargs)
         buffer.seek(0)
         await self.write_bytes_async(resolved_path, buffer.read())
+        _log_storage_event(
+            "storage.write",
+            backend_type=self.backend_type,
+            protocol=self.protocol,
+            operation="write_arrow",
+            mode="async",
+            path=resolved_path,
+        )
+
+    def stream_arrow_async(self, pattern: str, **kwargs: Any) -> AsyncIterator["ArrowRecordBatch"]:
+        """Stream Arrow record batches from storage asynchronously.
+
+        Args:
+            pattern: Glob pattern to match files.
+            **kwargs: Additional arguments passed to stream_arrow().
+
+        Returns:
+            AsyncIterator yielding Arrow record batches.
+        """
+        from sqlspec.storage.backends.base import AsyncArrowBatchIterator
 
-    def stream_arrow_async(self, pattern: str, **kwargs: Any) -> AsyncIterator[ArrowRecordBatch]:
         resolved_pattern = resolve_storage_path(pattern, self.base_path, self.protocol, strip_file_scheme=True)
-        return _AsyncArrowIterator(self, resolved_pattern, **kwargs)
+        return AsyncArrowBatchIterator(self.stream_arrow(resolved_pattern, **kwargs))
 
-    async def sign_async(self, path: str, expires_in: int = 3600, for_upload: bool = False) -> str:
-        """Generate a signed URL asynchronously."""
-        resolved_path = resolve_storage_path(path, self.base_path, self.protocol, strip_file_scheme=True)
-        return f"{self.store_uri}/{resolved_path}"
+    @overload
+    async def sign_async(self, paths: str, expires_in: int = 3600, for_upload: bool = False) -> str: ...
+
+    @overload
+    async def sign_async(self, paths: "list[str]", expires_in: int = 3600, for_upload: bool = False) -> "list[str]": ...
+
+    async def sign_async(
+        self, paths: "str | list[str]", expires_in: int = 3600, for_upload: bool = False
+    ) -> "str | list[str]":
+        """Generate signed URL(s) asynchronously.
+
+        Args:
+            paths: Single object path or list of paths to sign.
+            expires_in: URL expiration time in seconds (default: 3600, max: 604800 = 7 days).
+            for_upload: Whether the URL is for upload (PUT) vs download (GET).
+
+        Returns:
+            Single signed URL string if paths is a string, or list of signed URLs
+            if paths is a list. Preserves input type for convenience.
+
+        Raises:
+            NotImplementedError: If the backend protocol does not support signing.
+            ValueError: If expires_in exceeds maximum (604800 seconds).
+        """
+        import obstore as obs
+
+        signable_protocols = {"s3", "gs", "gcs", "az", "azure"}
+        if self.protocol not in signable_protocols:
+            msg = (
+                f"URL signing is not supported for protocol '{self.protocol}'. "
+                f"Only S3, GCS, and Azure backends support pre-signed URLs."
+            )
+            raise NotImplementedError(msg)
+
+        max_expires = 604800  # 7 days max per obstore/object_store limits
+        if expires_in > max_expires:
+            msg = f"expires_in cannot exceed {max_expires} seconds (7 days), got {expires_in}"
+            raise ValueError(msg)
+
+        from datetime import timedelta
+
+        method = "PUT" if for_upload else "GET"
+        expires_delta = timedelta(seconds=expires_in)
+
+        if isinstance(paths, str):
+            path_list = [paths]
+            is_single = True
+        else:
+            path_list = list(paths)
+            is_single = False
+
+        resolved_paths = [
+            resolve_storage_path(p, self.base_path, self.protocol, strip_file_scheme=True) for p in path_list
+        ]
+
+        try:
+            signed_urls: list[str] = await obs.sign_async(self.store, method, resolved_paths, expires_delta)  # type: ignore[call-overload]
+            return signed_urls[0] if is_single else signed_urls
+        except Exception as exc:
+            msg = f"Failed to generate signed URL(s) for {resolved_paths}"
+            raise StorageOperationFailedError(msg) from exc
