@@ -6,11 +6,15 @@ SQLSpec provides StructuredFormatter for JSON-formatted logs if desired.
 """
 
 import logging
-from contextvars import ContextVar
 from logging import LogRecord
-from typing import Any
+from typing import TYPE_CHECKING, Any, cast
 
 from sqlspec._serialization import encode_json
+from sqlspec.utils.correlation import CorrelationContext
+from sqlspec.utils.correlation import correlation_id_var as _correlation_id_var
+
+if TYPE_CHECKING:
+    from contextvars import ContextVar
 
 __all__ = (
     "SqlglotCommandFallbackFilter",
@@ -22,7 +26,36 @@ __all__ = (
     "suppress_erroneous_sqlglot_log_messages",
 )
 
-correlation_id_var: "ContextVar[str | None]" = ContextVar("correlation_id", default=None)
+_BASE_RECORD_KEYS: "set[str] | None" = None
+
+
+def _get_base_record_keys() -> "set[str]":
+    """Get base LogRecord keys lazily to avoid mypyc module-level dict issues."""
+    global _BASE_RECORD_KEYS
+    if _BASE_RECORD_KEYS is None:
+        _BASE_RECORD_KEYS = set(
+            logging.LogRecord(
+                name="sqlspec", level=logging.INFO, pathname="(unknown file)", lineno=0, msg="", args=(), exc_info=None
+            ).__dict__.keys()
+        )
+        _BASE_RECORD_KEYS.update({"message", "asctime"})
+    return _BASE_RECORD_KEYS
+
+
+correlation_id_var: "ContextVar[str | None]" = _correlation_id_var
+
+
+def _get_trace_context() -> "tuple[str | None, str | None]":
+    """Resolve trace context lazily to avoid import cycles.
+
+    Returns:
+        Tuple of (trace_id, span_id) or (None, None) if unavailable.
+    """
+    try:
+        from sqlspec.observability import get_trace_context
+    except Exception:
+        return (None, None)
+    return get_trace_context()
 
 
 def set_correlation_id(correlation_id: "str | None") -> None:
@@ -31,7 +64,7 @@ def set_correlation_id(correlation_id: "str | None") -> None:
     Args:
         correlation_id: The correlation ID to set, or None to clear
     """
-    correlation_id_var.set(correlation_id)
+    CorrelationContext.set(correlation_id)
 
 
 def get_correlation_id() -> "str | None":
@@ -40,7 +73,7 @@ def get_correlation_id() -> "str | None":
     Returns:
         The current correlation ID or None if not set
     """
-    return correlation_id_var.get()
+    return CorrelationContext.get()
 
 
 class StructuredFormatter(logging.Formatter):
@@ -65,11 +98,30 @@ class StructuredFormatter(logging.Formatter):
             "line": record.lineno,
         }
 
-        if correlation_id := get_correlation_id():
+        record_dict = record.__dict__
+        correlation_id = cast("str | None", record_dict.get("correlation_id")) or get_correlation_id()
+        if correlation_id:
             log_entry["correlation_id"] = correlation_id
+        trace_id = cast("str | None", record_dict.get("trace_id"))
+        span_id = cast("str | None", record_dict.get("span_id"))
+        if trace_id is None or span_id is None:
+            trace_id, span_id = _get_trace_context()
+        if trace_id:
+            log_entry["trace_id"] = trace_id
+        if span_id:
+            log_entry["span_id"] = span_id
 
-        if hasattr(record, "extra_fields"):
-            log_entry.update(record.extra_fields)  # pyright: ignore
+        extra_fields = record_dict.get("extra_fields")
+        if isinstance(extra_fields, dict):
+            log_entry.update(extra_fields)
+
+        extras = {
+            key: value
+            for key, value in record_dict.items()
+            if key not in _get_base_record_keys() and key not in {"extra_fields", "correlation_id"}
+        }
+        if extras:
+            log_entry.update(extras)
 
         if record.exc_info:
             log_entry["exception"] = self.formatException(record.exc_info)
@@ -95,23 +147,31 @@ class CorrelationIDFilter(logging.Filter):
 
 
 class SqlglotCommandFallbackFilter(logging.Filter):
-    """Filter to suppress sqlglot's confusing 'Falling back to Command' warning.
+    """Filter to suppress sqlglot warnings we consider benign.
 
-    This filter suppresses the warning message that sqlglot emits when it
-    encounters unsupported syntax and falls back to parsing as a Command.
-    This is expected behavior in SQLSpec and the warning is confusing to users.
+    - "Falling back to parsing as a 'Command'": emitted when sqlglot hits syntax it
+      intentionally downgrades; expected in SQLSpec usage.
+    - "Cannot traverse scope …": emitted by sqlglot's scope analysis in cases where
+      SQLSpec feeds partially constructed expressions; harmless for our flows.
     """
 
+    _suppressed_substrings = (
+        "falling back to parsing as a 'command'",
+        "cannot traverse scope",
+        "locking reads using 'for update/share' are not supported",
+    )
+
     def filter(self, record: LogRecord) -> bool:
-        """Suppress the 'Falling back to Command' warning message.
+        """Suppress known-safe sqlglot warnings.
 
         Args:
             record: The log record to evaluate
 
         Returns:
-            False if the record contains the fallback warning, True otherwise
+            False if the record message matches a suppressed pattern, True otherwise.
         """
-        return "Falling back to parsing as a 'Command'" not in record.getMessage()
+        message = record.getMessage().lower()
+        return not any(substr in message for substr in self._suppressed_substrings)
 
 
 def get_logger(name: "str | None" = None) -> logging.Logger:
@@ -146,9 +206,7 @@ def log_with_context(logger: logging.Logger, level: int, message: str, **extra_f
         message: Log message
         **extra_fields: Additional fields to include in structured logs
     """
-    record = logger.makeRecord(logger.name, level, "(unknown file)", 0, message, (), None)
-    record.extra_fields = extra_fields
-    logger.handle(record)
+    logger.log(level, message, extra={"extra_fields": extra_fields}, stacklevel=2)
 
 
 def suppress_erroneous_sqlglot_log_messages() -> None:
@@ -158,6 +216,7 @@ def suppress_erroneous_sqlglot_log_messages() -> None:
     about falling back to parsing as a Command. This is expected behavior
     in SQLSpec and the warning is confusing to users.
     """
-    sqlglot_logger = logging.getLogger("sqlglot")
-    if not any(isinstance(f, SqlglotCommandFallbackFilter) for f in sqlglot_logger.filters):
-        sqlglot_logger.addFilter(SqlglotCommandFallbackFilter())
+    for logger_name in ("sqlglot", "sqlglot.scope", "sqlglot.generator"):
+        sqlglot_logger = logging.getLogger(logger_name)
+        if not any(isinstance(f, SqlglotCommandFallbackFilter) for f in sqlglot_logger.filters):
+            sqlglot_logger.addFilter(SqlglotCommandFallbackFilter())
